@@ -20,6 +20,7 @@
  * should be used to speed up message creation.
  */
 
+#define PY_SSIZE_T_CLEAN
 #include "Python.h"
 
 #include "_cbsonmodule.h"
@@ -37,12 +38,6 @@ struct module_state {
 static struct module_state _state;
 #endif
 
-#if PY_MAJOR_VERSION >= 3
-#define BYTES_FORMAT_STRING "y#"
-#else
-#define BYTES_FORMAT_STRING "s#"
-#endif
-
 #define DOC_TOO_LARGE_FMT "BSON document too large (%d bytes)" \
                           " - the connected server supports" \
                           " BSON document sizes up to %ld bytes."
@@ -61,10 +56,21 @@ static PyObject* _error(char* name) {
     return error;
 }
 
+/* The same as buffer_write_bytes except that it also validates
+ * "size" will fit in an int.
+ * Returns 0 on failure */
+static int buffer_write_bytes_ssize_t(buffer_t buffer, const char* data, Py_ssize_t size) {
+    int downsize = _downcast_and_check(size, 0);
+    if (size == -1) {
+        return 0;
+    }
+    return buffer_write_bytes(buffer, data, downsize);
+}
+
 /* add a lastError message on the end of the buffer.
  * returns 0 on failure */
 static int add_last_error(PyObject* self, buffer_t buffer,
-                          int request_id, char* ns, int nslen,
+                          int request_id, char* ns, Py_ssize_t nslen,
                           codec_options_t* options, PyObject* args) {
     struct module_state *state = GETSTATE(self);
 
@@ -91,8 +97,7 @@ static int add_last_error(PyObject* self, buffer_t buffer,
                             "\xd4\x07\x00\x00"  /* opcode */
                             "\x00\x00\x00\x00", /* options */
                             12) ||
-        !buffer_write_bytes(buffer,
-                            ns, nslen) ||       /* database */
+        !buffer_write_bytes_ssize_t(buffer, ns, nslen) || /* database */
         !buffer_write_bytes(buffer,
                             ".$cmd\x00"         /* collection name */
                             "\x00\x00\x00\x00"  /* skip */
@@ -142,7 +147,7 @@ static int add_last_error(PyObject* self, buffer_t buffer,
 }
 
 static int init_insert_buffer(buffer_t buffer, int request_id, int options,
-                              const char* coll_name, int coll_name_len,
+                              const char* coll_name, Py_ssize_t coll_name_len,
                               int compress) {
     int length_location = 0;
     if (!compress) {
@@ -161,9 +166,9 @@ static int init_insert_buffer(buffer_t buffer, int request_id, int options,
         }
     }
     if (!buffer_write_int32(buffer, (int32_t)options) ||
-        !buffer_write_bytes(buffer,
-                            coll_name,
-                            coll_name_len + 1)) {
+        !buffer_write_bytes_ssize_t(buffer,
+                                    coll_name,
+                                    coll_name_len + 1)) {
         return -1;
     }
     return length_location;
@@ -177,7 +182,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     /* NOTE just using a random number as the request_id */
     int request_id = rand();
     char* collection_name = NULL;
-    int collection_name_length;
+    Py_ssize_t collection_name_length;
     PyObject* docs;
     PyObject* doc;
     PyObject* iterator;
@@ -293,7 +298,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "i", request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            max_size);
     destroy_codec_options(&options);
     buffer_free(buffer);
@@ -306,7 +311,7 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
 
     int request_id = rand();
     char* collection_name = NULL;
-    int collection_name_length;
+    Py_ssize_t collection_name_length;
     int before, cur_size, max_size = 0;
     PyObject* doc;
     PyObject* spec;
@@ -360,9 +365,9 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
                             "\xd1\x07\x00\x00"
                             "\x00\x00\x00\x00",
                             12) ||
-        !buffer_write_bytes(buffer,
-                            collection_name,
-                            collection_name_length + 1) ||
+        !buffer_write_bytes_ssize_t(buffer,
+                                    collection_name,
+                                    collection_name_length + 1) ||
         !buffer_write_int32(buffer, (int32_t)flags)) {
         destroy_codec_options(&options);
         buffer_free(buffer);
@@ -409,7 +414,7 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "i", request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            max_size);
     destroy_codec_options(&options);
     buffer_free(buffer);
@@ -424,7 +429,7 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     PyObject* cluster_time = NULL;
     unsigned int flags;
     char* collection_name = NULL;
-    int collection_name_length;
+    Py_ssize_t collection_name_length;
     int begin, cur_size, max_size = 0;
     int num_to_skip;
     int num_to_return;
@@ -434,7 +439,7 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     buffer_t buffer;
     int length_location, message_length;
     unsigned char check_keys = 0;
-    PyObject* result;
+    PyObject* result = NULL;
 
     if (!PyArg_ParseTuple(args, "Iet#iiOOO&|b",
                           &flags,
@@ -477,41 +482,29 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
             /* PyDict_GetItemString returns a borrowed reference. */
             Py_INCREF(cluster_time);
             if (-1 == PyMapping_DelItemString(query, "$clusterTime")) {
-                destroy_codec_options(&options);
-                PyMem_Free(collection_name);
-                return NULL;
+                goto fail;
             }
         }
     } else if (PyMapping_HasKeyString(query, "$clusterTime")) {
         cluster_time = PyMapping_GetItemString(query, "$clusterTime");
         if (!cluster_time
                 || -1 == PyMapping_DelItemString(query, "$clusterTime")) {
-            destroy_codec_options(&options);
-            PyMem_Free(collection_name);
-            return NULL;
+            goto fail;
         }
     }
     if (!buffer_write_int32(buffer, (int32_t)request_id) ||
         !buffer_write_bytes(buffer, "\x00\x00\x00\x00\xd4\x07\x00\x00", 8) ||
         !buffer_write_int32(buffer, (int32_t)flags) ||
-        !buffer_write_bytes(buffer, collection_name,
-                            collection_name_length + 1) ||
+        !buffer_write_bytes_ssize_t(buffer, collection_name,
+                                    collection_name_length + 1) ||
         !buffer_write_int32(buffer, (int32_t)num_to_skip) ||
         !buffer_write_int32(buffer, (int32_t)num_to_return)) {
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        Py_XDECREF(cluster_time);
-        return NULL;
+        goto fail;
     }
 
     begin = buffer_get_position(buffer);
     if (!write_dict(state->_cbson, buffer, query, check_keys, &options, 1)) {
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        Py_XDECREF(cluster_time);
-        return NULL;
+        goto fail;
     }
 
     /* back up a byte and write $clusterTime */
@@ -522,19 +515,11 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
         buffer_update_position(buffer, buffer_get_position(buffer) - 1);
         if (!write_pair(state->_cbson, buffer, "$clusterTime", 12, cluster_time,
                         0, &options, 1)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            Py_DECREF(cluster_time);
-            return NULL;
+            goto fail;
         }
 
         if (!buffer_write_bytes(buffer, &zero, 1)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            Py_DECREF(cluster_time);
-            return NULL;
+            goto fail;
         }
 
         length = buffer_get_position(buffer) - begin;
@@ -543,14 +528,10 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
         /* undo popping $clusterTime */
         if (-1 == PyMapping_SetItemString(
                 query, "$clusterTime", cluster_time)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            Py_DECREF(cluster_time);
-            return NULL;
+            goto fail;
         }
 
-        Py_DECREF(cluster_time);
+        Py_CLEAR(cluster_time);
     }
 
     max_size = buffer_get_position(buffer) - begin;
@@ -559,17 +540,12 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
         begin = buffer_get_position(buffer);
         if (!write_dict(state->_cbson, buffer, field_selector, 0,
                         &options, 1)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            return NULL;
+            goto fail;
         }
         cur_size = buffer_get_position(buffer) - begin;
         max_size = (cur_size > max_size) ? cur_size : max_size;
     }
 
-    PyMem_Free(collection_name);
-
     message_length = buffer_get_position(buffer) - length_location;
     buffer_write_int32_at_position(
         buffer, length_location, (int32_t)message_length);
@@ -577,10 +553,14 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "i", request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            max_size);
+
+fail:
+    PyMem_Free(collection_name);
     destroy_codec_options(&options);
     buffer_free(buffer);
+    Py_XDECREF(cluster_time);
     return result;
 }
 
@@ -588,7 +568,7 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     /* NOTE just using a random number as the request_id */
     int request_id = rand();
     char* collection_name = NULL;
-    int collection_name_length;
+    Py_ssize_t collection_name_length;
     int num_to_return;
     long long cursor_id;
     buffer_t buffer;
@@ -622,9 +602,9 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
                             "\x00\x00\x00\x00"
                             "\xd5\x07\x00\x00"
                             "\x00\x00\x00\x00", 12) ||
-        !buffer_write_bytes(buffer,
-                            collection_name,
-                            collection_name_length + 1) ||
+        !buffer_write_bytes_ssize_t(buffer,
+                                    collection_name,
+                                    collection_name_length + 1) ||
         !buffer_write_int32(buffer, (int32_t)num_to_return) ||
         !buffer_write_int64(buffer, (int64_t)cursor_id)) {
         buffer_free(buffer);
@@ -641,7 +621,7 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING, request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer));
+                           (Py_ssize_t)buffer_get_position(buffer));
     buffer_free(buffer);
     return result;
 }
@@ -659,7 +639,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     unsigned int flags;
     PyObject* command;
     char* identifier = NULL;
-    int identifier_length = 0;
+    Py_ssize_t identifier_length = 0;
     PyObject* docs;
     PyObject* doc;
     unsigned char check_keys = 0;
@@ -721,7 +701,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
         /* save space for payload 0 length */
         payload_one_length_location = buffer_save_space(buffer, 4);
         /* C string identifier */
-        if (!buffer_write_bytes(buffer, identifier, identifier_length + 1)) {
+        if (!buffer_write_bytes_ssize_t(buffer, identifier, identifier_length + 1)) {
             goto encodefail;
         }
         iterator = PyObject_GetIter(docs);
@@ -755,7 +735,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "ii", request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            total_size,
                            max_doc_size);
 encodefail:
@@ -788,7 +768,7 @@ _set_document_too_large(int size, long max) {
 static PyObject*
 _send_insert(PyObject* self, PyObject* ctx,
              PyObject* gle_args, buffer_t buffer,
-             char* coll_name, int coll_len, int request_id, int safe,
+             char* coll_name, Py_ssize_t coll_len, int request_id, int safe,
              codec_options_t* options, PyObject* to_publish, int compress) {
 
     if (safe) {
@@ -804,7 +784,7 @@ _send_insert(PyObject* self, PyObject* ctx,
                                "i" BYTES_FORMAT_STRING "iNOi",
                                request_id,
                                buffer_get_buffer(buffer),
-                               buffer_get_position(buffer),
+                               (Py_ssize_t)buffer_get_position(buffer),
                                0,
                                PyBool_FromLong((long)safe),
                                to_publish, compress);
@@ -817,7 +797,7 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
     int request_id = rand();
     int send_safe, flags = 0;
     int length_location, message_length;
-    int collection_name_length;
+    Py_ssize_t collection_name_length;
     int compress;
     char* collection_name = NULL;
     PyObject* docs;
@@ -1142,11 +1122,11 @@ _batched_op_msg(
     int size_location;
     int position;
     int length;
-    PyObject* max_bson_size_obj;
-    PyObject* max_write_batch_size_obj;
-    PyObject* max_message_size_obj;
-    PyObject* doc;
-    PyObject* iterator;
+    PyObject* max_bson_size_obj = NULL;
+    PyObject* max_write_batch_size_obj = NULL;
+    PyObject* max_message_size_obj = NULL;
+    PyObject* doc = NULL;
+    PyObject* iterator = NULL;
     char* flags = ack ? "\x00\x00\x00\x00" : "\x02\x00\x00\x00";
 
     max_bson_size_obj = PyObject_GetAttrString(ctx, "max_bson_size");
@@ -1209,7 +1189,7 @@ _batched_op_msg(
     case _INSERT:
         {
             if (!buffer_write_bytes(buffer, "documents\x00", 10))
-                goto cmdfail;
+                goto fail;
             break;
         }
     case _UPDATE:
@@ -1217,7 +1197,7 @@ _batched_op_msg(
             /* MongoDB does key validation for update. */
             check_keys = 0;
             if (!buffer_write_bytes(buffer, "updates\x00", 8))
-                goto cmdfail;
+                goto fail;
             break;
         }
     case _DELETE:
@@ -1225,7 +1205,7 @@ _batched_op_msg(
             /* Never check keys in a delete command. */
             check_keys = 0;
             if (!buffer_write_bytes(buffer, "deletes\x00", 8))
-                goto cmdfail;
+                goto fail;
             break;
         }
     default:
@@ -1255,7 +1235,7 @@ _batched_op_msg(
         int unacked_doc_too_large = 0;
         if (!write_dict(state->_cbson, buffer, doc, check_keys,
                         &options, 1)) {
-            goto cmditerfail;
+            goto fail;
         }
         cur_size = buffer_get_position(buffer) - cur_doc_begin;
 
@@ -1285,7 +1265,7 @@ _batched_op_msg(
                     Py_DECREF(DocumentTooLarge);
                 }
             }
-            goto cmditerfail;
+            goto fail;
         }
         /* We have enough data, return this batch. */
         if (buffer_get_position(buffer) > max_message_size) {
@@ -1294,10 +1274,11 @@ _batched_op_msg(
              * of the last document encoded.
              */
             buffer_update_position(buffer, cur_doc_begin);
+            Py_CLEAR(doc);
             break;
         }
         if (PyList_Append(to_publish, doc) < 0) {
-            goto cmditerfail;
+            goto fail;
         }
         Py_CLEAR(doc);
         idx += 1;
@@ -1306,10 +1287,10 @@ _batched_op_msg(
             break;
         }
     }
-    Py_DECREF(iterator);
+    Py_CLEAR(iterator);
 
     if (PyErr_Occurred()) {
-        goto cmdfail;
+        goto fail;
     }
 
     position = buffer_get_position(buffer);
@@ -1317,10 +1298,9 @@ _batched_op_msg(
     buffer_write_int32_at_position(buffer, size_location, (int32_t)length);
     return 1;
 
-cmditerfail:
+fail:
     Py_XDECREF(doc);
-    Py_DECREF(iterator);
-cmdfail:
+    Py_XDECREF(iterator);
     return 0;
 }
 
@@ -1369,7 +1349,7 @@ _cbson_encode_batched_op_msg(PyObject* self, PyObject* args) {
 
     result = Py_BuildValue(BYTES_FORMAT_STRING "O",
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            to_publish);
 fail:
     destroy_codec_options(&options);
@@ -1440,7 +1420,7 @@ _cbson_batched_op_msg(PyObject* self, PyObject* args) {
     buffer_write_int32_at_position(buffer, 4, (int32_t)request_id);
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "O", request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            to_publish);
 fail:
     destroy_codec_options(&options);
@@ -1453,7 +1433,7 @@ fail:
 
 static int
 _batched_write_command(
-        char* ns, int ns_len, unsigned char op, int check_keys,
+        char* ns, Py_ssize_t ns_len, unsigned char op, int check_keys,
         PyObject* command, PyObject* docs, PyObject* ctx,
         PyObject* to_publish, codec_options_t options,
         buffer_t buffer, struct module_state *state) {
@@ -1461,15 +1441,17 @@ _batched_write_command(
     long max_bson_size;
     long max_cmd_size;
     long max_write_batch_size;
+    long max_split_size;
     int idx = 0;
     int cmd_len_loc;
     int lst_len_loc;
     int position;
     int length;
-    PyObject* max_bson_size_obj;
-    PyObject* max_write_batch_size_obj;
-    PyObject* doc;
-    PyObject* iterator;
+    PyObject* max_bson_size_obj = NULL;
+    PyObject* max_write_batch_size_obj = NULL;
+    PyObject* max_split_size_obj = NULL;
+    PyObject* doc = NULL;
+    PyObject* iterator = NULL;
 
     max_bson_size_obj = PyObject_GetAttrString(ctx, "max_bson_size");
 #if PY_MAJOR_VERSION >= 3
@@ -1498,11 +1480,24 @@ _batched_write_command(
         return 0;
     }
 
+    // max_split_size is the size at which to perform a batch split.
+    // Normally this this value is equal to max_bson_size (16MiB). However,
+    // when auto encryption is enabled max_split_size is reduced to 2MiB.
+    max_split_size_obj = PyObject_GetAttrString(ctx, "max_split_size");
+#if PY_MAJOR_VERSION >= 3
+    max_split_size = PyLong_AsLong(max_split_size_obj);
+#else
+    max_split_size = PyInt_AsLong(max_split_size_obj);
+#endif
+    Py_XDECREF(max_split_size_obj);
+    if (max_split_size == -1) {
+        return 0;
+    }
+
     if (!buffer_write_bytes(buffer,
                             "\x00\x00\x00\x00", /* flags */
                             4) ||
-        !buffer_write_bytes(buffer,
-                            ns, ns_len + 1) ||  /* namespace */
+        !buffer_write_bytes_ssize_t(buffer, ns, ns_len + 1) || /* namespace */
         !buffer_write_bytes(buffer,
                             "\x00\x00\x00\x00"  /* skip */
                             "\xFF\xFF\xFF\xFF", /* limit (-1) */
@@ -1524,7 +1519,7 @@ _batched_write_command(
     case _INSERT:
         {
             if (!buffer_write_bytes(buffer, "documents\x00", 10))
-                goto cmdfail;
+                goto fail;
             break;
         }
     case _UPDATE:
@@ -1532,7 +1527,7 @@ _batched_write_command(
             /* MongoDB does key validation for update. */
             check_keys = 0;
             if (!buffer_write_bytes(buffer, "updates\x00", 8))
-                goto cmdfail;
+                goto fail;
             break;
         }
     case _DELETE:
@@ -1540,7 +1535,7 @@ _batched_write_command(
             /* Never check keys in a delete command. */
             check_keys = 0;
             if (!buffer_write_bytes(buffer, "deletes\x00", 8))
-                goto cmdfail;
+                goto fail;
             break;
         }
     default:
@@ -1575,69 +1570,72 @@ _batched_write_command(
         int cur_doc_begin;
         int cur_size;
         int enough_data = 0;
-        int enough_documents = 0;
         char key[16];
         INT2STRING(key, idx);
         if (!buffer_write_bytes(buffer, "\x03", 1) ||
             !buffer_write_bytes(buffer, key, (int)strlen(key) + 1)) {
-            goto cmditerfail;
+            goto fail;
         }
         cur_doc_begin = buffer_get_position(buffer);
         if (!write_dict(state->_cbson, buffer, doc,
                         check_keys, &options, 1)) {
-            goto cmditerfail;
+            goto fail;
         }
 
         /* We have enough data, return this batch.
          * max_cmd_size accounts for the two trailing null bytes.
          */
-        enough_data = (buffer_get_position(buffer) > max_cmd_size);
-        enough_documents = (idx >= max_write_batch_size);
-        if (enough_data || enough_documents) {
-            cur_size = buffer_get_position(buffer) - cur_doc_begin;
-
-            /* This single document is too large for the command. */
-            if (!idx) {
-                if (op == _INSERT) {
-                    _set_document_too_large(cur_size, max_bson_size);
-                } else {
-                    PyObject* DocumentTooLarge = _error("DocumentTooLarge");
-                    if (DocumentTooLarge) {
-                        /*
-                         * There's nothing intelligent we can say
-                         * about size for update and delete.
-                         */
-                        PyErr_Format(
-                            DocumentTooLarge,
-                            "%s command document too large",
-                            (op == _UPDATE) ? "update": "delete");
-                        Py_DECREF(DocumentTooLarge);
-                    }
+        cur_size = buffer_get_position(buffer) - cur_doc_begin;
+        /* This single document is too large for the command. */
+        if (cur_size > max_cmd_size) {
+            if (op == _INSERT) {
+                _set_document_too_large(cur_size, max_bson_size);
+            } else {
+                PyObject* DocumentTooLarge = _error("DocumentTooLarge");
+                if (DocumentTooLarge) {
+                    /*
+                     * There's nothing intelligent we can say
+                     * about size for update and delete.
+                     */
+                    PyErr_Format(
+                        DocumentTooLarge,
+                        "%s command document too large",
+                        (op == _UPDATE) ? "update": "delete");
+                    Py_DECREF(DocumentTooLarge);
                 }
-                goto cmditerfail;
             }
+            goto fail;
+        }
+        enough_data = (idx >= 1 &&
+                       (buffer_get_position(buffer) > max_split_size));
+        if (enough_data) {
             /*
              * Roll the existing buffer back to the beginning
              * of the last document encoded.
              */
             buffer_update_position(buffer, sub_doc_begin);
+            Py_CLEAR(doc);
             break;
         }
         if (PyList_Append(to_publish, doc) < 0) {
-            goto cmditerfail;
+            goto fail;
         }
         Py_CLEAR(doc);
         idx += 1;
+        /* We have enough documents, return this batch. */
+        if (idx == max_write_batch_size) {
+            break;
+        }
     }
-    Py_DECREF(iterator);
+    Py_CLEAR(iterator);
 
     if (PyErr_Occurred()) {
-        goto cmdfail;
+        goto fail;
     }
 
-    if (!buffer_write_bytes(buffer, "\x00\x00", 2))
-        goto cmdfail;
-
+    if (!buffer_write_bytes(buffer, "\x00\x00", 2)) {
+        goto fail;
+    }
 
     position = buffer_get_position(buffer);
     length = position - lst_len_loc - 1;
@@ -1646,10 +1644,9 @@ _batched_write_command(
     buffer_write_int32_at_position(buffer, cmd_len_loc, (int32_t)length);
     return 1;
 
-cmditerfail:
+fail:
     Py_XDECREF(doc);
-    Py_DECREF(iterator);
-cmdfail:
+    Py_XDECREF(iterator);
     return 0;
 }
 
@@ -1658,7 +1655,7 @@ _cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
     char *ns = NULL;
     unsigned char op;
     unsigned char check_keys;
-    int ns_len;
+    Py_ssize_t ns_len;
     PyObject* command;
     PyObject* docs;
     PyObject* ctx = NULL;
@@ -1701,7 +1698,7 @@ _cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
 
     result = Py_BuildValue(BYTES_FORMAT_STRING "O",
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            to_publish);
 fail:
     PyMem_Free(ns);
@@ -1716,7 +1713,7 @@ _cbson_batched_write_command(PyObject* self, PyObject* args) {
     char *ns = NULL;
     unsigned char op;
     unsigned char check_keys;
-    int ns_len;
+    Py_ssize_t ns_len;
     int request_id;
     int position;
     PyObject* command;
@@ -1776,7 +1773,7 @@ _cbson_batched_write_command(PyObject* self, PyObject* args) {
     buffer_write_int32_at_position(buffer, 4, (int32_t)request_id);
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "O", request_id,
                            buffer_get_buffer(buffer),
-                           buffer_get_position(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer),
                            to_publish);
 fail:
     PyMem_Free(ns);
