@@ -75,6 +75,19 @@ transactions on the same session can be executed in sequence.
 
 .. versionadded:: 3.7
 
+Sharded Transactions
+^^^^^^^^^^^^^^^^^^^^
+
+PyMongo 3.9 adds support for transactions on sharded clusters running MongoDB
+4.2. Sharded transactions have the same API as replica set transactions.
+When running a transaction against a sharded cluster, the session is
+pinned to the mongos server selected for the first operation in the
+transaction. All subsequent operations that are part of the same transaction
+are routed to the same mongos server. When the transaction is completed, by
+running either commitTransaction or abortTransaction, the session is unpinned.
+
+.. versionadded:: 3.9
+
 .. mongodoc:: transactions
 
 Classes
@@ -82,12 +95,14 @@ Classes
 """
 
 import collections
+import os
 import sys
 import uuid
 
 from bson.binary import Binary
 from bson.int64 import Int64
-from bson.py3compat import abc, reraise_instance
+from bson.py3compat import abc, integer_types, reraise_instance
+from bson.son import SON
 from bson.timestamp import Timestamp
 
 from pymongo import monotonic
@@ -95,8 +110,8 @@ from pymongo.errors import (ConfigurationError,
                             ConnectionFailure,
                             InvalidOperation,
                             OperationFailure,
+                            PyMongoError,
                             ServerSelectionTimeoutError,
-                            WriteConcernError,
                             WTimeoutError)
 from pymongo.helpers import _RETRYABLE_ERROR_CODES
 from pymongo.read_concern import ReadConcern
@@ -144,18 +159,35 @@ class TransactionOptions(object):
     """Options for :meth:`ClientSession.start_transaction`.
     
     :Parameters:
-      - `read_concern`: The :class:`~pymongo.read_concern.ReadConcern` to use
-        for this transaction.
-      - `write_concern`: The :class:`~pymongo.write_concern.WriteConcern` to
-        use for this transaction.
+      - `read_concern` (optional): The
+        :class:`~pymongo.read_concern.ReadConcern` to use for this transaction.
+        If ``None`` (the default) the :attr:`read_preference` of
+        the :class:`MongoClient` is used.
+      - `write_concern` (optional): The
+        :class:`~pymongo.write_concern.WriteConcern` to use for this
+        transaction. If ``None`` (the default) the :attr:`read_preference` of
+        the :class:`MongoClient` is used.
+      - `read_preference` (optional): The read preference to use. If
+        ``None`` (the default) the :attr:`read_preference` of this
+        :class:`MongoClient` is used. See :mod:`~pymongo.read_preferences`
+        for options. Transactions which read must use
+        :attr:`~pymongo.read_preferences.ReadPreference.PRIMARY`.
+      - `max_commit_time_ms` (optional): The maximum amount of time to allow a
+        single commitTransaction command to run. This option is an alias for
+        maxTimeMS option on the commitTransaction command. If ``None`` (the
+        default) maxTimeMS is not used.
+
+    .. versionchanged:: 3.9
+       Added the ``max_commit_time_ms`` option.
 
     .. versionadded:: 3.7
     """
     def __init__(self, read_concern=None, write_concern=None,
-                 read_preference=None):
+                 read_preference=None, max_commit_time_ms=None):
         self._read_concern = read_concern
         self._write_concern = write_concern
         self._read_preference = read_preference
+        self._max_commit_time_ms = max_commit_time_ms
         if read_concern is not None:
             if not isinstance(read_concern, ReadConcern):
                 raise TypeError("read_concern must be an instance of "
@@ -175,6 +207,10 @@ class TransactionOptions(object):
                 raise TypeError("%r is not valid for read_preference. See "
                                 "pymongo.read_preferences for valid "
                                 "options." % (read_preference,))
+        if max_commit_time_ms is not None:
+            if not isinstance(max_commit_time_ms, integer_types):
+                raise TypeError(
+                    "max_commit_time_ms must be an integer or None")
 
     @property
     def read_concern(self):
@@ -192,6 +228,14 @@ class TransactionOptions(object):
         """
         return self._read_preference
 
+    @property
+    def max_commit_time_ms(self):
+        """The maxTimeMS to use when running a commitTransaction command.
+
+        .. versionadded:: 3.9
+        """
+        return self._max_commit_time_ms
+
 
 def _validate_session_write_concern(session, write_concern):
     """Validate that an explicit session is not used with an unack'ed write.
@@ -224,7 +268,7 @@ class _TransactionContext(object):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        if self.__session._in_transaction:
+        if self.__session.in_transaction:
             if exc_val is None:
                 self.__session.commit_transaction()
             else:
@@ -245,11 +289,19 @@ class _Transaction(object):
     def __init__(self, opts):
         self.opts = opts
         self.state = _TxnState.NONE
-        self.transaction_id = 0
+        self.sharded = False
+        self.pinned_address = None
+        self.recovery_token = None
 
     def active(self):
         return self.state in (_TxnState.STARTING, _TxnState.IN_PROGRESS)
 
+    def reset(self):
+        self.state = _TxnState.NONE
+        self.sharded = False
+        self.pinned_address = None
+        self.recovery_token = None
+
 
 def _reraise_with_unknown_commit(exc):
     """Re-raise an exception with the UnknownTransactionCommitResult label."""
@@ -257,12 +309,29 @@ def _reraise_with_unknown_commit(exc):
     reraise_instance(exc, trace=sys.exc_info()[2])
 
 
+def _max_time_expired_error(exc):
+    """Return true if exc is a MaxTimeMSExpired error."""
+    return isinstance(exc, OperationFailure) and exc.code == 50
+
+
 # From the transactions spec, all the retryable writes errors plus
 # WriteConcernFailed.
 _UNKNOWN_COMMIT_ERROR_CODES = _RETRYABLE_ERROR_CODES | frozenset([
     64,    # WriteConcernFailed
+    50,    # MaxTimeMSExpired
 ])
 
+# From the Convenient API for Transactions spec, with_transaction must
+# halt retries after 120 seconds.
+# This limit is non-configurable and was chosen to be twice the 60 second
+# default value of MongoDB's `transactionLifetimeLimitSeconds` parameter.
+_WITH_TRANSACTION_RETRY_TIME_LIMIT = 120
+
+
+def _within_time_limit(start_time):
+    """Are we within the with_transaction retry limit?"""
+    return monotonic.time() - start_time < _WITH_TRANSACTION_RETRY_TIME_LIMIT
+
 
 class ClientSession(object):
     """A session for ordering sequential operations."""
@@ -288,7 +357,7 @@ class ClientSession(object):
     def _end_session(self, lock):
         if self._server_session is not None:
             try:
-                if self._in_transaction:
+                if self.in_transaction:
                     self.abort_transaction()
             finally:
                 self._client._return_server_session(self._server_session, lock)
@@ -346,29 +415,159 @@ class ClientSession(object):
             return val
         return getattr(self.client, name)
 
+    def with_transaction(self, callback, read_concern=None, write_concern=None,
+                         read_preference=None, max_commit_time_ms=None):
+        """Execute a callback in a transaction.
+
+        This method starts a transaction on this session, executes ``callback``
+        once, and then commits the transaction. For example::
+
+          def callback(session):
+              orders = session.client.db.orders
+              inventory = session.client.db.inventory
+              orders.insert_one({"sku": "abc123", "qty": 100}, session=session)
+              inventory.update_one({"sku": "abc123", "qty": {"$gte": 100}},
+                                   {"$inc": {"qty": -100}}, session=session)
+
+          with client.start_session() as session:
+              session.with_transaction(callback)
+
+        To pass arbitrary arguments to the ``callback``, wrap your callable
+        with a ``lambda`` like this::
+
+          def callback(session, custom_arg, custom_kwarg=None):
+              # Transaction operations...
+
+          with client.start_session() as session:
+              session.with_transaction(
+                  lambda s: callback(s, "custom_arg", custom_kwarg=1))
+
+        In the event of an exception, ``with_transaction`` may retry the commit
+        or the entire transaction, therefore ``callback`` may be invoked
+        multiple times by a single call to ``with_transaction``. Developers
+        should be mindful of this possiblity when writing a ``callback`` that
+        modifies application state or has any other side-effects.
+        Note that even when the ``callback`` is invoked multiple times,
+        ``with_transaction`` ensures that the transaction will be committed
+        at-most-once on the server.
+
+        The ``callback`` should not attempt to start new transactions, but
+        should simply run operations meant to be contained within a
+        transaction. The ``callback`` should also not commit the transaction;
+        this is handled automatically by ``with_transaction``. If the
+        ``callback`` does commit or abort the transaction without error,
+        however, ``with_transaction`` will return without taking further
+        action.
+
+        When ``callback`` raises an exception, ``with_transaction``
+        automatically aborts the current transaction. When ``callback`` or
+        :meth:`~ClientSession.commit_transaction` raises an exception that
+        includes the ``"TransientTransactionError"`` error label,
+        ``with_transaction`` starts a new transaction and re-executes
+        the ``callback``.
+
+        When :meth:`~ClientSession.commit_transaction` raises an exception with
+        the ``"UnknownTransactionCommitResult"`` error label,
+        ``with_transaction`` retries the commit until the result of the
+        transaction is known.
+
+        This method will cease retrying after 120 seconds has elapsed. This
+        timeout is not configurable and any exception raised by the
+        ``callback`` or by :meth:`ClientSession.commit_transaction` after the
+        timeout is reached will be re-raised. Applications that desire a
+        different timeout duration should not use this method.
+
+        :Parameters:
+          - `callback`: The callable ``callback`` to run inside a transaction.
+            The callable must accept a single argument, this session. Note,
+            under certain error conditions the callback may be run multiple
+            times.
+          - `read_concern` (optional): The
+            :class:`~pymongo.read_concern.ReadConcern` to use for this
+            transaction.
+          - `write_concern` (optional): The
+            :class:`~pymongo.write_concern.WriteConcern` to use for this
+            transaction.
+          - `read_preference` (optional): The read preference to use for this
+            transaction. If ``None`` (the default) the :attr:`read_preference`
+            of this :class:`Database` is used. See
+            :mod:`~pymongo.read_preferences` for options.
+
+        :Returns:
+          The return value of the ``callback``.
+
+        .. versionadded:: 3.9
+        """
+        start_time = monotonic.time()
+        while True:
+            self.start_transaction(
+                read_concern, write_concern, read_preference,
+                max_commit_time_ms)
+            try:
+                ret = callback(self)
+            except Exception as exc:
+                if self.in_transaction:
+                    self.abort_transaction()
+                if (isinstance(exc, PyMongoError) and
+                        exc.has_error_label("TransientTransactionError") and
+                        _within_time_limit(start_time)):
+                    # Retry the entire transaction.
+                    continue
+                raise
+
+            if not self.in_transaction:
+                # Assume callback intentionally ended the transaction.
+                return ret
+
+            while True:
+                try:
+                    self.commit_transaction()
+                except PyMongoError as exc:
+                    if (exc.has_error_label("UnknownTransactionCommitResult")
+                            and _within_time_limit(start_time)
+                            and not _max_time_expired_error(exc)):
+                        # Retry the commit.
+                        continue
+
+                    if (exc.has_error_label("TransientTransactionError") and
+                            _within_time_limit(start_time)):
+                        # Retry the entire transaction.
+                        break
+                    raise
+
+                # Commit succeeded.
+                return ret
+
     def start_transaction(self, read_concern=None, write_concern=None,
-                          read_preference=None):
+                          read_preference=None, max_commit_time_ms=None):
         """Start a multi-statement transaction.
 
         Takes the same arguments as :class:`TransactionOptions`.
 
+        .. versionchanged:: 3.9
+           Added the ``max_commit_time_ms`` option.
+
         .. versionadded:: 3.7
         """
         self._check_ended()
 
-        if self._in_transaction:
+        if self.in_transaction:
             raise InvalidOperation("Transaction already in progress")
 
         read_concern = self._inherit_option("read_concern", read_concern)
         write_concern = self._inherit_option("write_concern", write_concern)
         read_preference = self._inherit_option(
             "read_preference", read_preference)
+        if max_commit_time_ms is None:
+            opts = self.options.default_transaction_options
+            if opts:
+                max_commit_time_ms = opts.max_commit_time_ms
 
         self._transaction.opts = TransactionOptions(
-            read_concern, write_concern, read_preference)
+            read_concern, write_concern, read_preference, max_commit_time_ms)
+        self._transaction.reset()
         self._transaction.state = _TxnState.STARTING
-        self._server_session._transaction_id += 1
-        self._transaction.transaction_id = self._server_session.transaction_id
+        self._start_retryable_write()
         return _TransactionContext(self)
 
     def commit_transaction(self):
@@ -377,7 +576,7 @@ class ClientSession(object):
         .. versionadded:: 3.7
         """
         self._check_ended()
-
+        retry = False
         state = self._transaction.state
         if state is _TxnState.NONE:
             raise InvalidOperation("No transaction started")
@@ -388,9 +587,14 @@ class ClientSession(object):
         elif state is _TxnState.ABORTED:
             raise InvalidOperation(
                 "Cannot call commitTransaction after calling abortTransaction")
+        elif state is _TxnState.COMMITTED:
+            # We're explicitly retrying the commit, move the state back to
+            # "in progress" so that in_transaction returns true.
+            self._transaction.state = _TxnState.IN_PROGRESS
+            retry = True
 
         try:
-            self._finish_transaction_with_retry("commitTransaction")
+            self._finish_transaction_with_retry("commitTransaction", retry)
         except ConnectionFailure as exc:
             # We do not know if the commit was successfully applied on the
             # server or if it satisfied the provided write concern, set the
@@ -433,33 +637,29 @@ class ClientSession(object):
                 "Cannot call abortTransaction after calling commitTransaction")
 
         try:
-            self._finish_transaction_with_retry("abortTransaction")
+            self._finish_transaction_with_retry("abortTransaction", False)
         except (OperationFailure, ConnectionFailure):
             # The transactions spec says to ignore abortTransaction errors.
             pass
         finally:
             self._transaction.state = _TxnState.ABORTED
 
-    def _finish_transaction(self, command_name):
-        with self._client._socket_for_writes() as sock_info:
-            return self._client.admin._command(
-                sock_info,
-                command_name,
-                txnNumber=self._transaction.transaction_id,
-                autocommit=False,
-                session=self,
-                write_concern=self._transaction.opts.write_concern,
-                parse_write_concern_error=True)
+    def _finish_transaction_with_retry(self, command_name, explict_retry):
+        """Run commit or abort with one retry after any retryable error.
 
-    def _finish_transaction_with_retry(self, command_name):
+        :Parameters:
+          - `command_name`: Either "commitTransaction" or "abortTransaction".
+          - `explict_retry`: True when this is an explict commit retry attempt,
+            ie the application called session.commit_transaction() twice.
+        """
         # This can be refactored with MongoClient._retry_with_session.
         try:
-            return self._finish_transaction(command_name)
+            return self._finish_transaction(command_name, explict_retry)
         except ServerSelectionTimeoutError:
             raise
         except ConnectionFailure as exc:
             try:
-                return self._finish_transaction(command_name)
+                return self._finish_transaction(command_name, True)
             except ServerSelectionTimeoutError:
                 # Raise the original error so the application can infer that
                 # an attempt was made.
@@ -468,12 +668,40 @@ class ClientSession(object):
             if exc.code not in _RETRYABLE_ERROR_CODES:
                 raise
             try:
-                return self._finish_transaction(command_name)
+                return self._finish_transaction(command_name, True)
             except ServerSelectionTimeoutError:
                 # Raise the original error so the application can infer that
                 # an attempt was made.
                 raise exc
 
+    def _finish_transaction(self, command_name, retrying):
+        opts = self._transaction.opts
+        wc = opts.write_concern
+        cmd = SON([(command_name, 1)])
+        if command_name == "commitTransaction":
+            if opts.max_commit_time_ms:
+                cmd['maxTimeMS'] = opts.max_commit_time_ms
+
+            # Transaction spec says that after the initial commit attempt,
+            # subsequent commitTransaction commands should be upgraded to use
+            # w:"majority" and set a default value of 10 seconds for wtimeout.
+            if retrying:
+                wc_doc = wc.document
+                wc_doc["w"] = "majority"
+                wc_doc.setdefault("wtimeout", 10000)
+                wc = WriteConcern(**wc_doc)
+
+        if self._transaction.recovery_token:
+            cmd['recoveryToken'] = self._transaction.recovery_token
+
+        with self._client._socket_for_writes(self) as sock_info:
+            return self._client.admin._command(
+                sock_info,
+                cmd,
+                session=self,
+                write_concern=wc,
+                parse_write_concern_error=True)
+
     def _advance_cluster_time(self, cluster_time):
         """Internal cluster time helper."""
         if self._cluster_time is None:
@@ -518,19 +746,47 @@ class ClientSession(object):
                             "of bson.timestamp.Timestamp")
         self._advance_operation_time(operation_time)
 
+    def _process_response(self, reply):
+        """Process a response to a command that was run with this session."""
+        self._advance_cluster_time(reply.get('$clusterTime'))
+        self._advance_operation_time(reply.get('operationTime'))
+        if self.in_transaction and self._transaction.sharded:
+            recovery_token = reply.get('recoveryToken')
+            if recovery_token:
+                self._transaction.recovery_token = recovery_token
+
     @property
     def has_ended(self):
         """True if this session is finished."""
         return self._server_session is None
 
     @property
-    def _in_transaction(self):
-        """True if this session has an active multi-statement transaction."""
+    def in_transaction(self):
+        """True if this session has an active multi-statement transaction.
+
+        .. versionadded:: 3.10
+        """
         return self._transaction.active()
 
+    @property
+    def _pinned_address(self):
+        """The mongos address this transaction was created on."""
+        if self._transaction.active():
+            return self._transaction.pinned_address
+        return None
+
+    def _pin_mongos(self, server):
+        """Pin this session to the given mongos Server."""
+        self._transaction.sharded = True
+        self._transaction.pinned_address = server.description.address
+
+    def _unpin_mongos(self):
+        """Unpin this session from any pinned mongos address."""
+        self._transaction.pinned_address = None
+
     def _txn_read_preference(self):
         """Return read preference of this transaction or None."""
-        if self._in_transaction:
+        if self.in_transaction:
             return self._transaction.opts.read_preference
         return None
 
@@ -540,15 +796,14 @@ class ClientSession(object):
         self._server_session.last_use = monotonic.time()
         command['lsid'] = self._server_session.session_id
 
-        if not self._in_transaction:
-            self._transaction.state = _TxnState.NONE
+        if not self.in_transaction:
+            self._transaction.reset()
 
         if is_retryable:
-            self._server_session._transaction_id += 1
             command['txnNumber'] = self._server_session.transaction_id
             return
 
-        if self._in_transaction:
+        if self.in_transaction:
             if read_preference != ReadPreference.PRIMARY:
                 raise InvalidOperation(
                     'read preference in a transaction must be primary, not: '
@@ -574,17 +829,27 @@ class ClientSession(object):
             command['txnNumber'] = self._server_session.transaction_id
             command['autocommit'] = False
 
-    def _retry_transaction_id(self):
+    def _start_retryable_write(self):
         self._check_ended()
-        self._server_session.retry_transaction_id()
+        self._server_session.inc_transaction_id()
 
 
 class _ServerSession(object):
-    def __init__(self):
+    def __init__(self, pool_id):
         # Ensure id is type 4, regardless of CodecOptions.uuid_representation.
         self.session_id = {'id': Binary(uuid.uuid4().bytes, 4)}
         self.last_use = monotonic.time()
         self._transaction_id = 0
+        self.dirty = False
+        self.pool_id = pool_id
+
+    def mark_dirty(self):
+        """Mark this session as dirty.
+
+        A server session is marked dirty when a command fails with a network
+        error. Dirty sessions are later discarded from the server session pool.
+        """
+        self.dirty = True
 
     def timed_out(self, session_timeout_minutes):
         idle_seconds = monotonic.time() - self.last_use
@@ -597,8 +862,8 @@ class _ServerSession(object):
         """Positive 64-bit integer."""
         return Int64(self._transaction_id)
 
-    def retry_transaction_id(self):
-        self._transaction_id -= 1
+    def inc_transaction_id(self):
+        self._transaction_id += 1
 
 
 class _ServerSessionPool(collections.deque):
@@ -606,6 +871,14 @@ class _ServerSessionPool(collections.deque):
 
     This class is not thread-safe, access it while holding the Topology lock.
     """
+    def __init__(self, *args, **kwargs):
+        super(_ServerSessionPool, self).__init__(*args, **kwargs)
+        self.pool_id = 0
+
+    def reset(self):
+        self.pool_id += 1
+        self.clear()
+
     def pop_all(self):
         ids = []
         while self:
@@ -626,15 +899,18 @@ class _ServerSessionPool(collections.deque):
             if not s.timed_out(session_timeout_minutes):
                 return s
 
-        return _ServerSession()
+        return _ServerSession(self.pool_id)
 
     def return_server_session(self, server_session, session_timeout_minutes):
         self._clear_stale(session_timeout_minutes)
         if not server_session.timed_out(session_timeout_minutes):
-            self.appendleft(server_session)
+            self.return_server_session_no_lock(server_session)
 
     def return_server_session_no_lock(self, server_session):
-        self.appendleft(server_session)
+        # Discard sessions from an old pool to avoid duplicate sessions in the
+        # child process after a fork.
+        if server_session.pool_id == self.pool_id and not server_session.dirty:
+            self.appendleft(server_session)
 
     def _clear_stale(self, session_timeout_minutes):
         # Clear stale sessions. The least recently used are on the right.
