@@ -22,6 +22,8 @@ from bson.dbref import DBRef
 from bson.py3compat import iteritems, string_type, _unicode
 from bson.son import SON
 from pymongo import auth, common
+from pymongo.aggregation import _DatabaseAggregationCommand
+from pymongo.change_stream import DatabaseChangeStream
 from pymongo.collection import Collection
 from pymongo.command_cursor import CommandCursor
 from pymongo.errors import (CollectionInvalid,
@@ -31,11 +33,11 @@ from pymongo.errors import (CollectionInvalid,
 from pymongo.message import _first_batch
 from pymongo.read_preferences import ReadPreference
 from pymongo.son_manipulator import SONManipulator
-from pymongo.write_concern import WriteConcern
+from pymongo.write_concern import DEFAULT_WRITE_CONCERN
 
 
-_INDEX_REGEX = {"name": {"$regex": "^(?!.*\$)"}}
-_SYSTEM_FILTER = {"filter": {"name": {"$regex": "^(?!system\.)"}}}
+_INDEX_REGEX = {"name": {"$regex": r"^(?!.*\$)"}}
+_SYSTEM_FILTER = {"filter": {"name": {"$regex": r"^(?!system\.)"}}}
 
 
 def _check_name(name):
@@ -221,6 +223,46 @@ class Database(common.BaseObject):
         return [manipulator.__class__.__name__
                 for manipulator in self.__outgoing_copying_manipulators]
 
+    def with_options(self, codec_options=None, read_preference=None,
+                     write_concern=None, read_concern=None):
+        """Get a clone of this database changing the specified settings.
+
+          >>> db1.read_preference
+          Primary()
+          >>> from pymongo import ReadPreference
+          >>> db2 = db1.with_options(read_preference=ReadPreference.SECONDARY)
+          >>> db1.read_preference
+          Primary()
+          >>> db2.read_preference
+          Secondary(tag_sets=None)
+
+        :Parameters:
+          - `codec_options` (optional): An instance of
+            :class:`~bson.codec_options.CodecOptions`. If ``None`` (the
+            default) the :attr:`codec_options` of this :class:`Collection`
+            is used.
+          - `read_preference` (optional): The read preference to use. If
+            ``None`` (the default) the :attr:`read_preference` of this
+            :class:`Collection` is used. See :mod:`~pymongo.read_preferences`
+            for options.
+          - `write_concern` (optional): An instance of
+            :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
+            default) the :attr:`write_concern` of this :class:`Collection`
+            is used.
+          - `read_concern` (optional): An instance of
+            :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+            default) the :attr:`read_concern` of this :class:`Collection`
+            is used.
+
+        .. versionadded:: 3.8
+        """
+        return Database(self.client,
+                        self.__name,
+                        codec_options or self.codec_options,
+                        read_preference or self.read_preference,
+                        write_concern or self.write_concern,
+                        read_concern or self.read_concern)
+
     def __eq__(self, other):
         if isinstance(other, Database):
             return (self.__client == other.client and
@@ -300,15 +342,6 @@ class Database(common.BaseObject):
             self, name, False, codec_options, read_preference,
             write_concern, read_concern)
 
-    def _collection_default_options(self, name, **kargs):
-        """Get a Collection instance with the default settings."""
-        wc = (self.write_concern
-              if self.write_concern.acknowledged else WriteConcern())
-        return self.get_collection(
-            name, codec_options=DEFAULT_CODEC_OPTIONS,
-            read_preference=ReadPreference.PRIMARY,
-            write_concern=wc)
-
     def create_collection(self, name, codec_options=None,
                           read_preference=None, write_concern=None,
                           read_concern=None, session=None, **kwargs):
@@ -369,7 +402,8 @@ class Database(common.BaseObject):
            Removed deprecated argument: options
         """
         with self.__client._tmp_session(session) as s:
-            if name in self.collection_names(session=s):
+            if name in self.list_collection_names(
+                    filter={"name": name}, session=s):
                 raise CollectionInvalid("collection %s already exists" % name)
 
             return Collection(self, name, True, codec_options,
@@ -412,6 +446,165 @@ class Database(common.BaseObject):
             son = manipulator.transform_outgoing(son, collection)
         return son
 
+    def aggregate(self, pipeline, session=None, **kwargs):
+        """Perform a database-level aggregation.
+
+        See the `aggregation pipeline`_ documentation for a list of stages
+        that are supported.
+
+        Introduced in MongoDB 3.6.
+
+        .. code-block:: python
+
+           # Lists all operations currently running on the server.
+           with client.admin.aggregate([{"$currentOp": {}}]) as cursor:
+               for operation in cursor:
+                   print(operation)
+
+        All optional `aggregate command`_ parameters should be passed as
+        keyword arguments to this method. Valid options include, but are not
+        limited to:
+
+          - `allowDiskUse` (bool): Enables writing to temporary files. When set
+            to True, aggregation stages can write data to the _tmp subdirectory
+            of the --dbpath directory. The default is False.
+          - `maxTimeMS` (int): The maximum amount of time to allow the operation
+            to run in milliseconds.
+          - `batchSize` (int): The maximum number of documents to return per
+            batch. Ignored if the connected mongod or mongos does not support
+            returning aggregate results using a cursor.
+          - `collation` (optional): An instance of
+            :class:`~pymongo.collation.Collation`.
+
+        The :meth:`aggregate` method obeys the :attr:`read_preference` of this
+        :class:`Database`, except when ``$out`` or ``$merge`` are used, in
+        which case  :attr:`~pymongo.read_preferences.ReadPreference.PRIMARY`
+        is used.
+
+        .. note:: This method does not support the 'explain' option. Please
+           use :meth:`~pymongo.database.Database.command` instead.
+
+        .. note:: The :attr:`~pymongo.database.Database.write_concern` of
+           this collection is automatically applied to this operation.
+
+        :Parameters:
+          - `pipeline`: a list of aggregation pipeline stages
+          - `session` (optional): a
+            :class:`~pymongo.client_session.ClientSession`.
+          - `**kwargs` (optional): See list of options above.
+
+        :Returns:
+          A :class:`~pymongo.command_cursor.CommandCursor` over the result
+          set.
+
+        .. versionadded:: 3.9
+
+        .. _aggregation pipeline:
+            https://docs.mongodb.com/manual/reference/operator/aggregation-pipeline
+
+        .. _aggregate command:
+            https://docs.mongodb.com/manual/reference/command/aggregate
+        """
+        with self.client._tmp_session(session, close=False) as s:
+            cmd = _DatabaseAggregationCommand(
+                self, CommandCursor, pipeline, kwargs, session is not None,
+                user_fields={'cursor': {'firstBatch': 1}})
+            return self.client._retryable_read(
+                cmd.get_cursor, cmd.get_read_preference(s), s,
+                retryable=not cmd._performs_write)
+
+    def watch(self, pipeline=None, full_document=None, resume_after=None,
+              max_await_time_ms=None, batch_size=None, collation=None,
+              start_at_operation_time=None, session=None, start_after=None):
+        """Watch changes on this database.
+
+        Performs an aggregation with an implicit initial ``$changeStream``
+        stage and returns a
+        :class:`~pymongo.change_stream.DatabaseChangeStream` cursor which
+        iterates over changes on all collections in this database.
+
+        Introduced in MongoDB 4.0.
+
+        .. code-block:: python
+
+           with db.watch() as stream:
+               for change in stream:
+                   print(change)
+
+        The :class:`~pymongo.change_stream.DatabaseChangeStream` iterable
+        blocks until the next change document is returned or an error is
+        raised. If the
+        :meth:`~pymongo.change_stream.DatabaseChangeStream.next` method
+        encounters a network error when retrieving a batch from the server,
+        it will automatically attempt to recreate the cursor such that no
+        change events are missed. Any error encountered during the resume
+        attempt indicates there may be an outage and will be raised.
+
+        .. code-block:: python
+
+            try:
+                with db.watch(
+                        [{'$match': {'operationType': 'insert'}}]) as stream:
+                    for insert_change in stream:
+                        print(insert_change)
+            except pymongo.errors.PyMongoError:
+                # The ChangeStream encountered an unrecoverable error or the
+                # resume attempt failed to recreate the cursor.
+                logging.error('...')
+
+        For a precise description of the resume process see the
+        `change streams specification`_.
+
+        :Parameters:
+          - `pipeline` (optional): A list of aggregation pipeline stages to
+            append to an initial ``$changeStream`` stage. Not all
+            pipeline stages are valid after a ``$changeStream`` stage, see the
+            MongoDB documentation on change streams for the supported stages.
+          - `full_document` (optional): The fullDocument to pass as an option
+            to the ``$changeStream`` stage. Allowed values: 'updateLookup'.
+            When set to 'updateLookup', the change notification for partial
+            updates will include both a delta describing the changes to the
+            document, as well as a copy of the entire document that was
+            changed from some time after the change occurred.
+          - `resume_after` (optional): A resume token. If provided, the
+            change stream will start returning changes that occur directly
+            after the operation specified in the resume token. A resume token
+            is the _id value of a change document.
+          - `max_await_time_ms` (optional): The maximum time in milliseconds
+            for the server to wait for changes before responding to a getMore
+            operation.
+          - `batch_size` (optional): The maximum number of documents to return
+            per batch.
+          - `collation` (optional): The :class:`~pymongo.collation.Collation`
+            to use for the aggregation.
+          - `start_at_operation_time` (optional): If provided, the resulting
+            change stream will only return changes that occurred at or after
+            the specified :class:`~bson.timestamp.Timestamp`. Requires
+            MongoDB >= 4.0.
+          - `session` (optional): a
+            :class:`~pymongo.client_session.ClientSession`.
+          - `start_after` (optional): The same as `resume_after` except that
+            `start_after` can resume notifications after an invalidate event.
+            This option and `resume_after` are mutually exclusive.
+
+        :Returns:
+          A :class:`~pymongo.change_stream.DatabaseChangeStream` cursor.
+
+        .. versionchanged:: 3.9
+           Added the ``start_after`` parameter.
+
+        .. versionadded:: 3.7
+
+        .. mongodoc:: changeStreams
+
+        .. _change streams specification:
+            https://github.com/mongodb/specifications/blob/master/source/change-streams/change-streams.rst
+        """
+        return DatabaseChangeStream(
+            self, pipeline, full_document, resume_after, max_await_time_ms,
+            batch_size, collation, start_at_operation_time, session,
+            start_after)
+
     def _command(self, sock_info, command, slave_ok=False, value=1, check=True,
                  allowable_errors=None, read_preference=ReadPreference.PRIMARY,
                  codec_options=DEFAULT_CODEC_OPTIONS,
@@ -421,9 +614,6 @@ class Database(common.BaseObject):
         if isinstance(command, string_type):
             command = SON([(command, value)])
 
-        if sock_info.max_wire_version >= 5 and write_concern:
-            command['writeConcern'] = write_concern.document
-
         command.update(kwargs)
         with self.__client._tmp_session(session) as s:
             return sock_info.command(
@@ -434,12 +624,13 @@ class Database(common.BaseObject):
                 codec_options,
                 check,
                 allowable_errors,
+                write_concern=write_concern,
                 parse_write_concern_error=parse_write_concern_error,
                 session=s,
                 client=self.__client)
 
     def command(self, command, value=1, check=True,
-                allowable_errors=None, read_preference=ReadPreference.PRIMARY,
+                allowable_errors=None, read_preference=None,
                 codec_options=DEFAULT_CODEC_OPTIONS, session=None, **kwargs):
         """Issue a MongoDB command.
 
@@ -483,18 +674,25 @@ class Database(common.BaseObject):
             :class:`~pymongo.errors.OperationFailure` if there are any
           - `allowable_errors`: if `check` is ``True``, error messages
             in this list will be ignored by error-checking
-          - `read_preference`: The read preference for this operation.
-            See :mod:`~pymongo.read_preferences` for options.
+          - `read_preference` (optional): The read preference for this
+            operation. See :mod:`~pymongo.read_preferences` for options.
+            If the provided `session` is in a transaction, defaults to the
+            read preference configured for the transaction.
+            Otherwise, defaults to
+            :attr:`~pymongo.read_preferences.ReadPreference.PRIMARY`.
           - `codec_options`: A :class:`~bson.codec_options.CodecOptions`
             instance.
-          - `session` (optional): a
+          - `session` (optional): A
             :class:`~pymongo.client_session.ClientSession`.
           - `**kwargs` (optional): additional keyword arguments will
             be added to the command document before it is sent
 
-        .. note:: :meth:`command` does **not** obey :attr:`read_preference`
-           or :attr:`codec_options`. You must use the `read_preference` and
-           `codec_options` parameters instead.
+        .. note:: :meth:`command` does **not** obey this Database's
+           :attr:`read_preference` or :attr:`codec_options`. You must use the
+           `read_preference` and `codec_options` parameters instead.
+
+        .. note:: :meth:`command` does **not** apply any custom TypeDecoders
+           when decoding the command response.
 
         .. versionchanged:: 3.6
            Added ``session`` parameter.
@@ -525,16 +723,37 @@ class Database(common.BaseObject):
 
         .. mongodoc:: commands
         """
-        client = self.__client
-        with client._socket_for_reads(read_preference) as (sock_info, slave_ok):
+        if read_preference is None:
+            read_preference = ((session and session._txn_read_preference())
+                               or ReadPreference.PRIMARY)
+        with self.__client._socket_for_reads(
+                read_preference, session) as (sock_info, slave_ok):
             return self._command(sock_info, command, slave_ok, value,
                                  check, allowable_errors, read_preference,
                                  codec_options, session=session, **kwargs)
 
-    def _list_collections(self, sock_info, slave_okay, session=None, **kwargs):
+    def _retryable_read_command(self, command, value=1, check=True,
+                allowable_errors=None, read_preference=None,
+                codec_options=DEFAULT_CODEC_OPTIONS, session=None, **kwargs):
+        """Same as command but used for retryable read commands."""
+        if read_preference is None:
+            read_preference = ((session and session._txn_read_preference())
+                               or ReadPreference.PRIMARY)
+
+        def _cmd(session, server, sock_info, slave_ok):
+            return self._command(sock_info, command, slave_ok, value,
+                                 check, allowable_errors, read_preference,
+                                 codec_options, session=session, **kwargs)
+
+        return self.__client._retryable_read(
+            _cmd, read_preference, session)
+
+    def _list_collections(self, sock_info, slave_okay, session,
+                          read_preference, **kwargs):
         """Internal listCollections helper."""
 
-        coll = self["$cmd"]
+        coll = self.get_collection(
+            "$cmd", read_preference=read_preference)
         if sock_info.max_wire_version > 2:
             cmd = SON([("listCollections", 1),
                        ("cursor", {})])
@@ -542,7 +761,9 @@ class Database(common.BaseObject):
             with self.__client._tmp_session(
                     session, close=False) as tmp_session:
                 cursor = self._command(
-                    sock_info, cmd, slave_okay, session=tmp_session)["cursor"]
+                    sock_info, cmd, slave_okay,
+                    read_preference=read_preference,
+                    session=tmp_session)["cursor"]
                 return CommandCursor(
                     coll,
                     cursor,
@@ -565,12 +786,14 @@ class Database(common.BaseObject):
             cursor = self._command(sock_info, cmd, slave_okay)["cursor"]
             return CommandCursor(coll, cursor, sock_info.address)
 
-    def list_collections(self, session=None, **kwargs):
+    def list_collections(self, session=None, filter=None, **kwargs):
         """Get a cursor over the collectons of this database.
 
         :Parameters:
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
+          - `filter` (optional):  A query document to filter the list of
+            collections returned from the listCollections command.
           - `**kwargs` (optional): Optional parameters of the
             `listCollections command
             <https://docs.mongodb.com/manual/reference/command/listCollections/>`_
@@ -582,26 +805,60 @@ class Database(common.BaseObject):
 
         .. versionadded:: 3.6
         """
-        with self.__client._socket_for_reads(
-                ReadPreference.PRIMARY) as (sock_info, slave_okay):
-            return self._list_collections(
-                sock_info, slave_okay, session=session, **kwargs)
+        if filter is not None:
+            kwargs['filter'] = filter
+        read_pref = ((session and session._txn_read_preference())
+                     or ReadPreference.PRIMARY)
 
-    def list_collection_names(self, session=None):
+        def _cmd(session, server, sock_info, slave_okay):
+            return self._list_collections(
+                sock_info, slave_okay, session, read_preference=read_pref,
+                **kwargs)
+
+        return self.__client._retryable_read(
+            _cmd, read_pref, session)
+
+    def list_collection_names(self, session=None, filter=None, **kwargs):
         """Get a list of all the collection names in this database.
 
+        For example, to list all non-system collections::
+
+            filter = {"name": {"$regex": r"^(?!system\\.)"}}
+            db.list_collection_names(filter=filter)
+
         :Parameters:
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
+          - `filter` (optional):  A query document to filter the list of
+            collections returned from the listCollections command.
+          - `**kwargs` (optional): Optional parameters of the
+            `listCollections command
+            <https://docs.mongodb.com/manual/reference/command/listCollections/>`_
+            can be passed as keyword arguments to this method. The supported
+            options differ by server version.
+
+        .. versionchanged:: 3.8
+           Added the ``filter`` and ``**kwargs`` parameters.
 
         .. versionadded:: 3.6
         """
+        if filter is None:
+            kwargs["nameOnly"] = True
+        else:
+            # The enumerate collections spec states that "drivers MUST NOT set
+            # nameOnly if a filter specifies any keys other than name."
+            common.validate_is_mapping("filter", filter)
+            kwargs["filter"] = filter
+            if not filter or (len(filter) == 1 and "name" in filter):
+                kwargs["nameOnly"] = True
+
         return [result["name"]
-                for result in self.list_collections(session=session)]
+                for result in self.list_collections(session=session, **kwargs)]
 
     def collection_names(self, include_system_collections=True,
                          session=None):
-        """Get a list of all the collection names in this database.
+        """**DEPRECATED**: Get a list of all the collection names in this
+        database.
 
         :Parameters:
           - `include_system_collections` (optional): if ``False`` list
@@ -609,12 +866,19 @@ class Database(common.BaseObject):
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
 
+        .. versionchanged:: 3.7
+           Deprecated. Use :meth:`list_collection_names` instead.
+
         .. versionchanged:: 3.6
            Added ``session`` parameter.
         """
+        warnings.warn("collection_names is deprecated. Use "
+                      "list_collection_names instead.",
+                      DeprecationWarning, stacklevel=2)
         kws = {} if include_system_collections else _SYSTEM_FILTER
         return [result["name"]
-                for result in self.list_collections(session=session, **kws)]
+                for result in self.list_collections(session=session,
+                                                    nameOnly=True, **kws)]
 
     def drop_collection(self, name_or_collection, session=None):
         """Drop a collection.
@@ -647,12 +911,11 @@ class Database(common.BaseObject):
 
         self.__client._purge_index(self.__name, name)
 
-        with self.__client._socket_for_reads(
-                ReadPreference.PRIMARY) as (sock_info, slave_ok):
+        with self.__client._socket_for_writes(session) as sock_info:
             return self._command(
-                sock_info, 'drop', slave_ok, _unicode(name),
+                sock_info, 'drop', value=_unicode(name),
                 allowable_errors=['ns not found'],
-                write_concern=self.write_concern,
+                write_concern=self._write_concern_for(session),
                 parse_write_concern_error=True,
                 session=session)
 
@@ -716,8 +979,41 @@ class Database(common.BaseObject):
 
         return result
 
+    def _current_op(self, include_all=False, session=None):
+        """Helper for running $currentOp."""
+        cmd = SON([("currentOp", 1), ("$all", include_all)])
+        with self.__client._socket_for_writes(session) as sock_info:
+            if sock_info.max_wire_version >= 4:
+                return self.__client.admin._command(
+                    sock_info, cmd, codec_options=self.codec_options,
+                    session=session)
+            else:
+                spec = {"$all": True} if include_all else {}
+                return _first_batch(sock_info, "admin", "$cmd.sys.inprog",
+                                    spec, -1, True, self.codec_options,
+                                    ReadPreference.PRIMARY, cmd,
+                                    self.client._event_listeners)
+
     def current_op(self, include_all=False, session=None):
-        """Get information on operations currently running.
+        """**DEPRECATED**: Get information on operations currently running.
+
+        Starting with MongoDB 3.6 this helper is obsolete. The functionality
+        provided by this helper is available in MongoDB 3.6+ using the
+        `$currentOp aggregation pipeline stage`_, which can be used with
+        :meth:`aggregate`. Note that, while this helper can only return
+        a single document limited to a 16MB result, :meth:`aggregate`
+        returns a cursor avoiding that limitation.
+
+        Users of MongoDB versions older than 3.6 can use the `currentOp command`_
+        directly::
+
+          # MongoDB 3.2 and 3.4
+          client.admin.command("currentOp")
+
+        Or query the "inprog" virtual collection::
+
+          # MongoDB 2.6 and 3.0
+          client.admin["$cmd.sys.inprog"].find_one()
 
         :Parameters:
           - `include_all` (optional): if ``True`` also list currently
@@ -725,21 +1021,19 @@ class Database(common.BaseObject):
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
 
+        .. versionchanged:: 3.9
+           Deprecated.
+
         .. versionchanged:: 3.6
            Added ``session`` parameter.
+
+        .. _$currentOp aggregation pipeline stage: https://docs.mongodb.com/manual/reference/operator/aggregation/currentOp/
+        .. _currentOp command: https://docs.mongodb.com/manual/reference/command/currentOp/
         """
-        cmd = SON([("currentOp", 1), ("$all", include_all)])
-        with self.__client._socket_for_writes() as sock_info:
-            if sock_info.max_wire_version >= 4:
-                with self.__client._tmp_session(session) as s:
-                    return sock_info.command("admin", cmd, session=s,
-                                             client=self.__client)
-            else:
-                spec = {"$all": True} if include_all else {}
-                return _first_batch(sock_info, "admin", "$cmd.sys.inprog",
-                                    spec, -1, True, self.codec_options,
-                                    ReadPreference.PRIMARY, cmd,
-                                    self.client._event_listeners)
+        warnings.warn("current_op() is deprecated. See the documentation for "
+                      "more information",
+                      DeprecationWarning, stacklevel=2)
+        return self._current_op(include_all, session)
 
     def profiling_level(self, session=None):
         """Get the database's current profiling level.
@@ -934,19 +1228,17 @@ class Database(common.BaseObject):
 
             opts["roles"] = [self._default_role(read_only)]
 
-        elif read_only:
+        if read_only:
             warnings.warn("The read_only option is deprecated in MongoDB "
                           ">= 2.6, use 'roles' instead", DeprecationWarning)
 
         if password is not None:
-            # We always salt and hash client side.
             if "digestPassword" in kwargs:
                 raise ConfigurationError("The digestPassword option is not "
                                          "supported via add_user. Please use "
                                          "db.command('createUser', ...) "
                                          "instead for this option.")
-            opts["pwd"] = auth._password_digest(name, password)
-            opts["digestPassword"] = False
+            opts["pwd"] = password
 
         # Don't send {} as writeConcern.
         if self.write_concern.acknowledged and self.write_concern.document:
@@ -994,6 +1286,9 @@ class Database(common.BaseObject):
         .. _updateUser: https://docs.mongodb.com/manual/reference/command/updateUser/
         .. _dropUser: https://docs.mongodb.com/manual/reference/command/createUser/
 
+        .. warning:: Never create or modify users over an insecure network without
+          the use of TLS. See :doc:`/examples/tls` for more information.
+
         :Parameters:
           - `name`: the name of the user to create
           - `password` (optional): the password of the user to create. Can not
@@ -1006,6 +1301,9 @@ class Database(common.BaseObject):
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
 
+        .. versionchanged:: 3.7
+           Added support for SCRAM-SHA-256 users with MongoDB 4.0 and later.
+
         .. versionchanged:: 3.6
            Added ``session`` parameter. Deprecated add_user.
 
@@ -1113,15 +1411,20 @@ class Database(common.BaseObject):
             Not used with GSSAPI or MONGODB-X509 authentication.
           - `source` (optional): the database to authenticate on. If not
             specified the current database is used.
-          - `mechanism` (optional): See
-            :data:`~pymongo.auth.MECHANISMS` for options.
-            By default, use SCRAM-SHA-1 with MongoDB 3.0 and later,
-            MONGODB-CR (MongoDB Challenge Response protocol) for older servers.
+          - `mechanism` (optional): See :data:`~pymongo.auth.MECHANISMS` for
+            options. If no mechanism is specified, PyMongo automatically uses
+            MONGODB-CR when connected to a pre-3.0 version of MongoDB,
+            SCRAM-SHA-1 when connected to MongoDB 3.0 through 3.6, and
+            negotiates the mechanism to use (SCRAM-SHA-1 or SCRAM-SHA-256) when
+            connected to MongoDB 4.0+.
           - `authMechanismProperties` (optional): Used to specify
             authentication mechanism specific options. To specify the service
             name for GSSAPI authentication pass
             authMechanismProperties='SERVICE_NAME:<service name>'
 
+        .. versionchanged:: 3.7
+           Added support for SCRAM-SHA-256 with MongoDB 4.0 and later.
+
         .. versionchanged:: 3.5
            Deprecated. Authenticating multiple users conflicts with support for
            logical sessions in MongoDB 3.6. To authenticate as multiple users,
@@ -1156,10 +1459,11 @@ class Database(common.BaseObject):
 
         credentials = auth._build_credentials_tuple(
             mechanism,
-            source or self.name,
+            source,
             name,
             password,
-            validated_options)
+            validated_options,
+            self.name)
 
         self.client._cache_credentials(
             self.name,
@@ -1255,7 +1559,7 @@ class SystemJS(object):
 
         if not database.write_concern.acknowledged:
             database = database.client.get_database(
-                database.name, write_concern=WriteConcern())
+                database.name, write_concern=DEFAULT_WRITE_CONCERN)
         # can't just assign it since we've overridden __setattr__
         object.__setattr__(self, "_db", database)
 
