@@ -34,13 +34,17 @@ try:
 except ImportError:
     _SELECT_ERROR = OSError
 
+from bson import _decode_all_selective
+from bson.py3compat import PY3
+
 from pymongo import helpers, message
 from pymongo.common import MAX_MESSAGE_SIZE
+from pymongo.compression_support import decompress, _NO_COMPRESSION
 from pymongo.errors import (AutoReconnect,
                             NotMasterError,
                             OperationFailure,
                             ProtocolError)
-from pymongo.message import _OpReply
+from pymongo.message import _UNPACK_REPLY
 
 
 _UNPACK_HEADER = struct.Struct("<iiii").unpack
@@ -52,13 +56,17 @@ def command(sock, dbname, spec, slave_ok, is_mongos,
             check_keys=False, listeners=None, max_bson_size=None,
             read_concern=None,
             parse_write_concern_error=False,
-            collation=None):
+            collation=None,
+            compression_ctx=None,
+            use_op_msg=False,
+            unacknowledged=False,
+            user_fields=None):
     """Execute a command over the socket, or raise socket.error.
 
     :Parameters:
       - `sock`: a raw socket instance
       - `dbname`: name of the database on which to run the command
-      - `spec`: a command document as a dict, SON, or mapping object
+      - `spec`: a command document as an ordered dict type, eg SON.
       - `slave_ok`: whether to set the SlaveOkay wire protocol bit
       - `is_mongos`: are we connected to a mongos?
       - `read_preference`: a read preference
@@ -75,6 +83,12 @@ def command(sock, dbname, spec, slave_ok, is_mongos,
       - `parse_write_concern_error`: Whether to parse the ``writeConcernError``
         field in the command response.
       - `collation`: The collation for this command.
+      - `compression_ctx`: optional compression Context.
+      - `use_op_msg`: True if we should use OP_MSG.
+      - `unacknowledged`: True if this is an unacknowledged command.
+      - `user_fields` (optional): Response fields that should be decoded
+        using the TypeDecoders from codec_options, passed to
+        bson._decode_all_selective.
     """
     name = next(iter(spec))
     ns = dbname + '.$cmd'
@@ -82,9 +96,9 @@ def command(sock, dbname, spec, slave_ok, is_mongos,
 
     # Publish the original command document, perhaps with lsid and $clusterTime.
     orig = spec
-    if is_mongos:
+    if is_mongos and not use_op_msg:
         spec = message._maybe_add_read_preference(spec, read_preference)
-    if read_concern:
+    if read_concern and not (session and session.in_transaction):
         if read_concern.level:
             spec['readConcern'] = read_concern.document
         if (session and session.options.causal_consistency
@@ -98,8 +112,30 @@ def command(sock, dbname, spec, slave_ok, is_mongos,
     if publish:
         start = datetime.datetime.now()
 
-    request_id, msg, size = message.query(flags, ns, 0, -1, spec,
-                                          None, codec_options, check_keys)
+    if compression_ctx and name.lower() in _NO_COMPRESSION:
+        compression_ctx = None
+
+    if (client and client._encrypter and
+            not client._encrypter._bypass_auto_encryption):
+        spec = orig = client._encrypter.encrypt(
+            dbname, spec, check_keys, codec_options)
+        # We already checked the keys, no need to do it again.
+        check_keys = False
+
+    if use_op_msg:
+        flags = 2 if unacknowledged else 0
+        request_id, msg, size, max_doc_size = message._op_msg(
+            flags, spec, dbname, read_preference, slave_ok, check_keys,
+            codec_options, ctx=compression_ctx)
+        # If this is an unacknowledged write then make sure the encoded doc(s)
+        # are small enough, otherwise rely on the server to return an error.
+        if (unacknowledged and max_bson_size is not None and
+                max_doc_size > max_bson_size):
+            message._raise_document_too_large(name, size, max_bson_size)
+    else:
+        request_id, msg, size = message.query(
+            flags, ns, 0, -1, spec, None, codec_options, check_keys,
+            compression_ctx)
 
     if (max_bson_size is not None
             and size > max_bson_size + message._COMMAND_OVERHEAD):
@@ -113,16 +149,22 @@ def command(sock, dbname, spec, slave_ok, is_mongos,
 
     try:
         sock.sendall(msg)
-        reply = receive_message(sock, request_id)
-        unpacked_docs = reply.unpack_response(codec_options=codec_options)
+        if use_op_msg and unacknowledged:
+            # Unacknowledged, fake a successful command response.
+            reply = None
+            response_doc = {"ok": 1}
+        else:
+            reply = receive_message(sock, request_id)
+            unpacked_docs = reply.unpack_response(
+                codec_options=codec_options, user_fields=user_fields)
 
-        response_doc = unpacked_docs[0]
-        if client:
-            client._receive_cluster_time(response_doc, session)
-        if check:
-            helpers._check_command_response(
-                response_doc, None, allowable_errors,
-                parse_write_concern_error=parse_write_concern_error)
+            response_doc = unpacked_docs[0]
+            if client:
+                client._process_response(response_doc, session)
+            if check:
+                helpers._check_command_response(
+                    response_doc, None, allowable_errors,
+                    parse_write_concern_error=parse_write_concern_error)
     except Exception as exc:
         if publish:
             duration = (datetime.datetime.now() - start) + encoding_duration
@@ -137,17 +179,21 @@ def command(sock, dbname, spec, slave_ok, is_mongos,
         duration = (datetime.datetime.now() - start) + encoding_duration
         listeners.publish_command_success(
             duration, response_doc, name, request_id, address)
+
+    if client and client._encrypter and reply:
+        decrypted = client._encrypter.decrypt(reply.raw_command_response())
+        response_doc = _decode_all_selective(decrypted, codec_options,
+                                             user_fields)[0]
+
     return response_doc
 
+_UNPACK_COMPRESSION_HEADER = struct.Struct("<iiB").unpack
 
 def receive_message(sock, request_id, max_message_size=MAX_MESSAGE_SIZE):
     """Receive a raw BSON message or raise socket.error."""
     # Ignore the response's request id.
     length, _, response_to, op_code = _UNPACK_HEADER(
         _receive_data_on_socket(sock, 16))
-    if op_code != _OpReply.OP_CODE:
-        raise ProtocolError("Got opcode %r but expected "
-                            "%r" % (op_code, _OpReply.OP_CODE))
     # No request_id for exhaust cursor "getMore".
     if request_id is not None:
         if request_id != response_to:
@@ -159,26 +205,64 @@ def receive_message(sock, request_id, max_message_size=MAX_MESSAGE_SIZE):
     if length > max_message_size:
         raise ProtocolError("Message length (%r) is larger than server max "
                             "message size (%r)" % (length, max_message_size))
+    if op_code == 2012:
+        op_code, _, compressor_id = _UNPACK_COMPRESSION_HEADER(
+            _receive_data_on_socket(sock, 9))
+        data = decompress(
+            _receive_data_on_socket(sock, length - 25), compressor_id)
+    else:
+        data = _receive_data_on_socket(sock, length - 16)
 
-    return _OpReply.unpack(_receive_data_on_socket(sock, length - 16))
+    try:
+        unpack_reply = _UNPACK_REPLY[op_code]
+    except KeyError:
+        raise ProtocolError("Got opcode %r but expected "
+                            "%r" % (op_code, _UNPACK_REPLY.keys()))
+    return unpack_reply(data)
 
 
-def _receive_data_on_socket(sock, length):
-    msg = b""
-    while length:
-        try:
-            chunk = sock.recv(length)
-        except (IOError, OSError) as exc:
-            if _errno_from_exception(exc) == errno.EINTR:
-                continue
-            raise
-        if chunk == b"":
-            raise AutoReconnect("connection closed")
+# memoryview was introduced in Python 2.7 but we only use it on Python 3
+# because before 2.7.4 the struct module did not support memoryview:
+# https://bugs.python.org/issue10212.
+# In Jython, using slice assignment on a memoryview results in a
+# NullPointerException.
+if not PY3:
+    def _receive_data_on_socket(sock, length):
+        buf = bytearray(length)
+        i = 0
+        while length:
+            try:
+                chunk = sock.recv(length)
+            except (IOError, OSError) as exc:
+                if _errno_from_exception(exc) == errno.EINTR:
+                    continue
+                raise
+            if chunk == b"":
+                raise AutoReconnect("connection closed")
 
-        length -= len(chunk)
-        msg += chunk
+            buf[i:i + len(chunk)] = chunk
+            i += len(chunk)
+            length -= len(chunk)
 
-    return msg
+        return bytes(buf)
+else:
+    def _receive_data_on_socket(sock, length):
+        buf = bytearray(length)
+        mv = memoryview(buf)
+        bytes_read = 0
+        while bytes_read < length:
+            try:
+                chunk_length = sock.recv_into(mv[bytes_read:])
+            except (IOError, OSError) as exc:
+                if _errno_from_exception(exc) == errno.EINTR:
+                    continue
+                raise
+            if chunk_length == 0:
+                raise AutoReconnect("connection closed")
+
+            bytes_read += chunk_length
+
+        return mv
 
 
 def _errno_from_exception(exc):
