@@ -1,4 +1,4 @@
-# Copyright 2015 MongoDB, Inc.
+# Copyright 2015-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,37 +14,16 @@
 
 """Operation class definitions."""
 
-from pymongo.common import validate_boolean, validate_is_mapping
+from pymongo.common import validate_boolean, validate_is_mapping, validate_list
+from pymongo.collation import validate_collation_or_none
 from pymongo.helpers import _gen_index_name, _index_document, _index_list
 
 
-class _WriteOp(object):
-    """Private base class for all write operations."""
-
-    __slots__ = ("_filter", "_doc", "_upsert")
-
-    def __init__(self, filter=None, doc=None, upsert=None):
-        if filter is not None:
-            validate_is_mapping("filter", filter)
-        if upsert is not None:
-            validate_boolean("upsert", upsert)
-        self._filter = filter
-        self._doc = doc
-        self._upsert = upsert
-
-    def __eq__(self, other):
-        if type(other) == type(self):
-            return (other._filter, other._doc, other._upsert) == \
-                   (self._filter, self._doc, self._upsert)
-        return NotImplemented
-
-    def __ne__(self, other):
-        return not self == other
-
-
-class InsertOne(_WriteOp):
+class InsertOne(object):
     """Represents an insert_one operation."""
 
+    __slots__ = ("_doc",)
+
     def __init__(self, document):
         """Create an InsertOne instance.
 
@@ -54,7 +33,7 @@ class InsertOne(_WriteOp):
           - `document`: The document to insert. If the document is missing an
             _id field one will be added.
         """
-        super(InsertOne, self).__init__(doc=document)
+        self._doc = document
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
@@ -63,53 +42,103 @@ class InsertOne(_WriteOp):
     def __repr__(self):
         return "InsertOne(%r)" % (self._doc,)
 
+    def __eq__(self, other):
+        if type(other) == type(self):
+            return other._doc == self._doc
+        return NotImplemented
 
-class DeleteOne(_WriteOp):
+    def __ne__(self, other):
+        return not self == other
+
+
+class DeleteOne(object):
     """Represents a delete_one operation."""
 
-    def __init__(self, filter):
+    __slots__ = ("_filter", "_collation")
+
+    def __init__(self, filter, collation=None):
         """Create a DeleteOne instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
 
         :Parameters:
           - `filter`: A query that matches the document to delete.
+          - `collation` (optional): An instance of
+            :class:`~pymongo.collation.Collation`. This option is only
+            supported on MongoDB 3.4 and above.
+
+        .. versionchanged:: 3.5
+           Added the `collation` option.
         """
-        super(DeleteOne, self).__init__(filter)
+        if filter is not None:
+            validate_is_mapping("filter", filter)
+        self._filter = filter
+        self._collation = collation
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_delete(self._filter, 1)
+        bulkobj.add_delete(self._filter, 1, collation=self._collation)
 
     def __repr__(self):
-        return "DeleteOne(%r)" % (self._filter,)
+        return "DeleteOne(%r, %r)" % (self._filter, self._collation)
+
+    def __eq__(self, other):
+        if type(other) == type(self):
+            return ((other._filter, other._collation) ==
+                    (self._filter, self._collation))
+        return NotImplemented
+
+    def __ne__(self, other):
+        return not self == other
 
 
-class DeleteMany(_WriteOp):
+class DeleteMany(object):
     """Represents a delete_many operation."""
 
-    def __init__(self, filter):
+    __slots__ = ("_filter", "_collation")
+
+    def __init__(self, filter, collation=None):
         """Create a DeleteMany instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
 
         :Parameters:
           - `filter`: A query that matches the documents to delete.
+          - `collation` (optional): An instance of
+            :class:`~pymongo.collation.Collation`. This option is only
+            supported on MongoDB 3.4 and above.
+
+        .. versionchanged:: 3.5
+           Added the `collation` option.
         """
-        super(DeleteMany, self).__init__(filter)
+        if filter is not None:
+            validate_is_mapping("filter", filter)
+        self._filter = filter
+        self._collation = collation
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_delete(self._filter, 0)
+        bulkobj.add_delete(self._filter, 0, collation=self._collation)
 
     def __repr__(self):
-        return "DeleteMany(%r)" % (self._filter,)
+        return "DeleteMany(%r, %r)" % (self._filter, self._collation)
+
+    def __eq__(self, other):
+        if type(other) == type(self):
+            return ((other._filter, other._collation) ==
+                    (self._filter, self._collation))
+        return NotImplemented
+
+    def __ne__(self, other):
+        return not self == other
 
 
-class ReplaceOne(_WriteOp):
+class ReplaceOne(object):
     """Represents a replace_one operation."""
 
-    def __init__(self, filter, replacement, upsert=False):
+    __slots__ = ("_filter", "_doc", "_upsert", "_collation")
+
+    def __init__(self, filter, replacement, upsert=False, collation=None):
         """Create a ReplaceOne instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -119,23 +148,86 @@ class ReplaceOne(_WriteOp):
           - `replacement`: The new document.
           - `upsert` (optional): If ``True``, perform an insert if no documents
             match the filter.
+          - `collation` (optional): An instance of
+            :class:`~pymongo.collation.Collation`. This option is only
+            supported on MongoDB 3.4 and above.
+
+        .. versionchanged:: 3.5
+           Added the `collation` option.
         """
-        super(ReplaceOne, self).__init__(filter, replacement, upsert)
+        if filter is not None:
+            validate_is_mapping("filter", filter)
+        if upsert is not None:
+            validate_boolean("upsert", upsert)
+        self._filter = filter
+        self._doc = replacement
+        self._upsert = upsert
+        self._collation = collation
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_replace(self._filter, self._doc, self._upsert)
+        bulkobj.add_replace(self._filter, self._doc, self._upsert,
+                            collation=self._collation)
+
+    def __eq__(self, other):
+        if type(other) == type(self):
+            return (
+                (other._filter, other._doc, other._upsert, other._collation) ==
+                (self._filter, self._doc, self._upsert, self._collation))
+        return NotImplemented
+
+    def __ne__(self, other):
+        return not self == other
 
     def __repr__(self):
-        return "ReplaceOne(%r, %r, %r)" % (self._filter,
-                                           self._doc,
-                                           self._upsert)
+        return "%s(%r, %r, %r, %r)" % (
+            self.__class__.__name__, self._filter, self._doc, self._upsert,
+            self._collation)
 
 
-class UpdateOne(_WriteOp):
+class _UpdateOp(object):
+    """Private base class for update operations."""
+
+    __slots__ = ("_filter", "_doc", "_upsert", "_collation", "_array_filters")
+
+    def __init__(self, filter, doc, upsert, collation, array_filters):
+        if filter is not None:
+            validate_is_mapping("filter", filter)
+        if upsert is not None:
+            validate_boolean("upsert", upsert)
+        if array_filters is not None:
+            validate_list("array_filters", array_filters)
+        self._filter = filter
+        self._doc = doc
+        self._upsert = upsert
+        self._collation = collation
+        self._array_filters = array_filters
+
+    def __eq__(self, other):
+        if type(other) == type(self):
+            return (
+                (other._filter, other._doc, other._upsert, other._collation,
+                 other._array_filters) ==
+                (self._filter, self._doc, self._upsert, self._collation,
+                 self._array_filters))
+        return NotImplemented
+
+    def __ne__(self, other):
+        return not self == other
+
+    def __repr__(self):
+        return "%s(%r, %r, %r, %r, %r)" % (
+            self.__class__.__name__, self._filter, self._doc, self._upsert,
+            self._collation, self._array_filters)
+
+
+class UpdateOne(_UpdateOp):
     """Represents an update_one operation."""
 
-    def __init__(self, filter, update, upsert=False):
+    __slots__ = ()
+
+    def __init__(self, filter, update, upsert=False, collation=None,
+                 array_filters=None):
         """Represents an update_one operation.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -145,23 +237,36 @@ class UpdateOne(_WriteOp):
           - `update`: The modifications to apply.
           - `upsert` (optional): If ``True``, perform an insert if no documents
             match the filter.
+          - `collation` (optional): An instance of
+            :class:`~pymongo.collation.Collation`. This option is only
+            supported on MongoDB 3.4 and above.
+          - `array_filters` (optional): A list of filters specifying which
+            array elements an update should apply. Requires MongoDB 3.6+.
+
+        .. versionchanged:: 3.9
+           Added the ability to accept a pipeline as the `update`.
+        .. versionchanged:: 3.6
+           Added the `array_filters` option.
+        .. versionchanged:: 3.5
+           Added the `collation` option.
         """
-        super(UpdateOne, self).__init__(filter, update, upsert)
+        super(UpdateOne, self).__init__(filter, update, upsert, collation,
+                                        array_filters)
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_update(self._filter, self._doc, False, self._upsert)
-
-    def __repr__(self):
-        return "UpdateOne(%r, %r, %r)" % (self._filter,
-                                          self._doc,
-                                          self._upsert)
+        bulkobj.add_update(self._filter, self._doc, False, self._upsert,
+                           collation=self._collation,
+                           array_filters=self._array_filters)
 
 
-class UpdateMany(_WriteOp):
+class UpdateMany(_UpdateOp):
     """Represents an update_many operation."""
 
-    def __init__(self, filter, update, upsert=False):
+    __slots__ = ()
+
+    def __init__(self, filter, update, upsert=False, collation=None,
+                 array_filters=None):
         """Create an UpdateMany instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -171,17 +276,27 @@ class UpdateMany(_WriteOp):
           - `update`: The modifications to apply.
           - `upsert` (optional): If ``True``, perform an insert if no documents
             match the filter.
+          - `collation` (optional): An instance of
+            :class:`~pymongo.collation.Collation`. This option is only
+            supported on MongoDB 3.4 and above.
+          - `array_filters` (optional): A list of filters specifying which
+            array elements an update should apply. Requires MongoDB 3.6+.
+
+        .. versionchanged:: 3.9
+           Added the ability to accept a pipeline as the `update`.
+        .. versionchanged:: 3.6
+           Added the `array_filters` option.
+        .. versionchanged:: 3.5
+           Added the `collation` option.
         """
-        super(UpdateMany, self).__init__(filter, update, upsert)
+        super(UpdateMany, self).__init__(filter, update, upsert, collation,
+                                         array_filters)
 
     def _add_to_bulk(self, bulkobj):
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_update(self._filter, self._doc, True, self._upsert)
-
-    def __repr__(self):
-        return "UpdateMany(%r, %r, %r)" % (self._filter,
-                                           self._doc,
-                                           self._upsert)
+        bulkobj.add_update(self._filter, self._doc, True, self._upsert,
+                           collation=self._collation,
+                           array_filters=self._array_filters)
 
 
 class IndexModel(object):
@@ -223,13 +338,16 @@ class IndexModel(object):
             this collection after <int> seconds. The indexed field must
             be a UTC datetime or the data will not expire.
           - `partialFilterExpression`: A document that specifies a filter for
-            a partial index.
+            a partial index. Requires server version >= 3.2.
+          - `collation`: An instance of :class:`~pymongo.collation.Collation`
+            that specifies the collation to use in MongoDB >= 3.4.
+          - `wildcardProjection`: Allows users to include or exclude specific
+            field paths from a `wildcard index`_ using the { "$**" : 1} key
+            pattern. Requires server version >= 4.2.
 
         See the MongoDB documentation for a full list of supported options by
         server version.
 
-        .. note:: `partialFilterExpression` requires server version **>= 3.2**
-
         :Parameters:
           - `keys`: a single key or a list of (key, direction)
             pairs specifying the index to create
@@ -239,12 +357,17 @@ class IndexModel(object):
 
         .. versionchanged:: 3.2
             Added partialFilterExpression to support partial indexes.
+
+        .. _wildcard index: https://docs.mongodb.com/master/core/index-wildcard/#wildcard-index-core
         """
         keys = _index_list(keys)
         if "name" not in kwargs:
             kwargs["name"] = _gen_index_name(keys)
         kwargs["key"] = _index_document(keys)
+        collation = validate_collation_or_none(kwargs.pop('collation', None))
         self.__document = kwargs
+        if collation is not None:
+            self.__document['collation'] = collation
 
     @property
     def document(self):
