@@ -1,4 +1,4 @@
-# Copyright 2012-2015 MongoDB, Inc.
+# Copyright 2012-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -118,7 +118,7 @@ def main(options, mode, url):
     start_time = time.time()
     errors = 0
     if mode == 'parallel':
-        nrequests_per_thread = options.nrequests / options.nthreads
+        nrequests_per_thread = options.nrequests // options.nthreads
 
         if options.verbose:
             print (
