@@ -26,15 +26,19 @@ except ImportError:
 
 sys.path[0:0] = [""]
 
-from pymongo import MongoClient
+from pymongo import MongoClient, monitoring
 from pymongo.auth import HAVE_KERBEROS, _build_credentials_tuple
 from pymongo.errors import OperationFailure
 from pymongo.read_preferences import ReadPreference
+from pymongo.saslprep import HAVE_STRINGPREP
 from test import client_context, SkipTest, unittest, Version
 from test.utils import (delay,
                         ignore_deprecations,
+                        single_client,
+                        rs_or_single_client,
                         rs_or_single_client_noauth,
-                        single_client_noauth)
+                        single_client_noauth,
+                        WhiteListEventListener)
 
 # YOU MUST RUN KINIT BEFORE RUNNING GSSAPI TESTS ON UNIX.
 GSSAPI_HOST = os.environ.get('GSSAPI_HOST')
@@ -73,6 +77,31 @@ class AutoAuthenticateThread(threading.Thread):
         self.success = True
 
 
+class DBAuthenticateThread(threading.Thread):
+    """Used in testing threaded authentication.
+
+    This does db.test.find_one() with a 1-second delay to ensure it must
+    check out and authenticate multiple sockets from the pool concurrently.
+
+    :Parameters:
+      `db`: An auth-protected db with a 'test' collection containing one
+      document.
+    """
+
+    def __init__(self, db, username, password):
+        super(DBAuthenticateThread, self).__init__()
+        self.db = db
+        self.username = username
+        self.password = password
+        self.success = False
+
+    def run(self):
+        self.db.authenticate(self.username, self.password)
+        assert self.db.test.find_one({'$where': delay(1)}) is not None
+        self.success = True
+
+
+
 class TestGSSAPI(unittest.TestCase):
 
     @classmethod
@@ -95,19 +124,19 @@ class TestGSSAPI(unittest.TestCase):
     def test_credentials_hashing(self):
         # GSSAPI credentials are properly hashed.
         creds0 = _build_credentials_tuple(
-            'GSSAPI', '', 'user', 'pass', {})
+            'GSSAPI', None, 'user', 'pass', {}, None)
 
         creds1 = _build_credentials_tuple(
-            'GSSAPI', '', 'user', 'pass',
-            {'authmechanismproperties': {'SERVICE_NAME': 'A'}})
+            'GSSAPI', None, 'user', 'pass',
+            {'authmechanismproperties': {'SERVICE_NAME': 'A'}}, None)
 
         creds2 = _build_credentials_tuple(
-            'GSSAPI', '', 'user', 'pass',
-            {'authmechanismproperties': {'SERVICE_NAME': 'A'}})
+            'GSSAPI', None, 'user', 'pass',
+            {'authmechanismproperties': {'SERVICE_NAME': 'A'}}, None)
 
         creds3 = _build_credentials_tuple(
-            'GSSAPI', '', 'user', 'pass',
-            {'authmechanismproperties': {'SERVICE_NAME': 'B'}})
+            'GSSAPI', None, 'user', 'pass',
+            {'authmechanismproperties': {'SERVICE_NAME': 'B'}}, None)
 
         self.assertEqual(1, len(set([creds1, creds2])))
         self.assertEqual(3, len(set([creds0, creds1, creds2, creds3])))
@@ -166,11 +195,11 @@ class TestGSSAPI(unittest.TestCase):
                                      authMechanism='GSSAPI',
                                      replicaSet=set_name)
 
-                client[GSSAPI_DB].collection_names()
+                client[GSSAPI_DB].list_collection_names()
 
                 uri = uri + '&replicaSet=%s' % (str(set_name),)
                 client = MongoClient(uri)
-                client[GSSAPI_DB].collection_names()
+                client[GSSAPI_DB].list_collection_names()
 
             # With authMechanismProperties
             client = MongoClient(GSSAPI_HOST,
@@ -181,11 +210,11 @@ class TestGSSAPI(unittest.TestCase):
                                  authMechanismProperties=self.mech_properties,
                                  replicaSet=set_name)
 
-            client[GSSAPI_DB].collection_names()
+            client[GSSAPI_DB].list_collection_names()
 
             mech_uri = mech_uri + '&replicaSet=%s' % (str(set_name),)
             client = MongoClient(mech_uri)
-            client[GSSAPI_DB].collection_names()
+            client[GSSAPI_DB].list_collection_names()
 
     @ignore_deprecations
     def test_gssapi_threaded(self):
@@ -205,7 +234,7 @@ class TestGSSAPI(unittest.TestCase):
         # multiple sockets from the pool concurrently, proving that
         # auto-authentication works with GSSAPI.
         collection = db.test
-        if collection.count() == 0:
+        if not collection.count_documents({}):
             try:
                 collection.drop()
                 collection.insert_one({'_id': 1})
@@ -366,6 +395,267 @@ class TestSCRAMSHA1(unittest.TestCase):
             db.command('dbstats')
 
 
+class TestSCRAM(unittest.TestCase):
+
+    @client_context.require_auth
+    @client_context.require_version_min(3, 7, 2)
+    def setUp(self):
+        self._SENSITIVE_COMMANDS = monitoring._SENSITIVE_COMMANDS
+        monitoring._SENSITIVE_COMMANDS = set([])
+        self.listener = WhiteListEventListener("saslStart")
+
+    def tearDown(self):
+        monitoring._SENSITIVE_COMMANDS = self._SENSITIVE_COMMANDS
+        client_context.client.testscram.command("dropAllUsersFromDatabase")
+        client_context.client.drop_database("testscram")
+
+    @ignore_deprecations
+    def test_scram(self):
+        host, port = client_context.host, client_context.port
+
+        client_context.create_user(
+            'testscram',
+            'sha1',
+            'pwd',
+            roles=['dbOwner'],
+            mechanisms=['SCRAM-SHA-1'])
+
+        client_context.create_user(
+            'testscram',
+            'sha256',
+            'pwd',
+            roles=['dbOwner'],
+            mechanisms=['SCRAM-SHA-256'])
+
+        client_context.create_user(
+            'testscram',
+            'both',
+            'pwd',
+            roles=['dbOwner'],
+            mechanisms=['SCRAM-SHA-1', 'SCRAM-SHA-256'])
+
+        client = rs_or_single_client_noauth(
+            event_listeners=[self.listener])
+        self.assertTrue(
+            client.testscram.authenticate('sha1', 'pwd'))
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+        self.assertTrue(
+            client.testscram.authenticate(
+                'sha1', 'pwd', mechanism='SCRAM-SHA-1'))
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+        self.assertRaises(
+            OperationFailure,
+            client.testscram.authenticate,
+            'sha1', 'pwd', mechanism='SCRAM-SHA-256')
+
+        self.assertTrue(
+            client.testscram.authenticate('sha256', 'pwd'))
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+        self.assertTrue(
+            client.testscram.authenticate(
+                'sha256', 'pwd', mechanism='SCRAM-SHA-256'))
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+        self.assertRaises(
+            OperationFailure,
+            client.testscram.authenticate,
+            'sha256', 'pwd', mechanism='SCRAM-SHA-1')
+
+        self.listener.results.clear()
+        self.assertTrue(
+            client.testscram.authenticate('both', 'pwd'))
+        started = self.listener.results['started'][0]
+        self.assertEqual(started.command.get('mechanism'), 'SCRAM-SHA-256')
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+        self.assertTrue(
+            client.testscram.authenticate(
+                'both', 'pwd', mechanism='SCRAM-SHA-256'))
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+        self.assertTrue(
+            client.testscram.authenticate(
+                'both', 'pwd', mechanism='SCRAM-SHA-1'))
+        client.testscram.command('dbstats')
+        client.testscram.logout()
+
+        self.assertRaises(
+            OperationFailure,
+            client.testscram.authenticate,
+            'not-a-user', 'pwd')
+
+        if HAVE_STRINGPREP:
+            # Test the use of SASLprep on passwords. For example,
+            # saslprep(u'\u2136') becomes u'IV' and saslprep(u'I\u00ADX')
+            # becomes u'IX'. SASLprep is only supported when the standard
+            # library provides stringprep.
+            client_context.create_user(
+                'testscram',
+                u'\u2168',
+                u'\u2163',
+                roles=['dbOwner'],
+                mechanisms=['SCRAM-SHA-256'])
+
+            client_context.create_user(
+                'testscram',
+                u'IX',
+                u'IX',
+                roles=['dbOwner'],
+                mechanisms=['SCRAM-SHA-256'])
+
+            self.assertTrue(
+                client.testscram.authenticate(u'\u2168', u'\u2163'))
+            client.testscram.command('dbstats')
+            client.testscram.logout()
+            self.assertTrue(
+                client.testscram.authenticate(
+                    u'\u2168', u'\u2163', mechanism='SCRAM-SHA-256'))
+            client.testscram.command('dbstats')
+            client.testscram.logout()
+            self.assertTrue(
+                client.testscram.authenticate(u'\u2168', u'IV'))
+            client.testscram.command('dbstats')
+            client.testscram.logout()
+
+            self.assertTrue(
+                client.testscram.authenticate(u'IX', u'I\u00ADX'))
+            client.testscram.command('dbstats')
+            client.testscram.logout()
+            self.assertTrue(
+                client.testscram.authenticate(
+                    u'IX', u'I\u00ADX', mechanism='SCRAM-SHA-256'))
+            client.testscram.command('dbstats')
+            client.testscram.logout()
+            self.assertTrue(
+                client.testscram.authenticate(u'IX', u'IX'))
+            client.testscram.command('dbstats')
+            client.testscram.logout()
+
+            client = rs_or_single_client_noauth(
+                u'mongodb://\u2168:\u2163@%s:%d/testscram' % (host, port))
+            client.testscram.command('dbstats')
+            client = rs_or_single_client_noauth(
+                u'mongodb://\u2168:IV@%s:%d/testscram' % (host, port))
+            client.testscram.command('dbstats')
+
+            client = rs_or_single_client_noauth(
+                u'mongodb://IX:I\u00ADX@%s:%d/testscram' % (host, port))
+            client.testscram.command('dbstats')
+            client = rs_or_single_client_noauth(
+                u'mongodb://IX:IX@%s:%d/testscram' % (host, port))
+            client.testscram.command('dbstats')
+
+        self.listener.results.clear()
+        client = rs_or_single_client_noauth(
+            'mongodb://both:pwd@%s:%d/testscram' % (host, port),
+            event_listeners=[self.listener])
+        client.testscram.command('dbstats')
+        started = self.listener.results['started'][0]
+        self.assertEqual(started.command.get('mechanism'), 'SCRAM-SHA-256')
+
+        client = rs_or_single_client_noauth(
+            'mongodb://both:pwd@%s:%d/testscram?authMechanism=SCRAM-SHA-1'
+            % (host, port))
+        client.testscram.command('dbstats')
+
+        client = rs_or_single_client_noauth(
+            'mongodb://both:pwd@%s:%d/testscram?authMechanism=SCRAM-SHA-256'
+            % (host, port))
+        client.testscram.command('dbstats')
+
+        if client_context.is_rs:
+            uri = ('mongodb://both:pwd@%s:%d/testscram'
+                   '?replicaSet=%s' % (host, port,
+                                       client_context.replica_set_name))
+            client = single_client_noauth(uri)
+            client.testscram.command('dbstats')
+            db = client.get_database(
+                'testscram', read_preference=ReadPreference.SECONDARY)
+            db.command('dbstats')
+
+    def test_cache(self):
+        client = single_client()
+        # Force authentication.
+        client.admin.command('ismaster')
+        all_credentials = client._MongoClient__all_credentials
+        credentials = all_credentials.get('admin')
+        cache = credentials.cache
+        self.assertIsNotNone(cache)
+        data = cache.data
+        self.assertIsNotNone(data)
+        self.assertEqual(len(data), 4)
+        ckey, skey, salt, iterations = data
+        self.assertIsInstance(ckey, bytes)
+        self.assertIsInstance(skey, bytes)
+        self.assertIsInstance(salt, bytes)
+        self.assertIsInstance(iterations, int)
+
+        pool = next(iter(client._topology._servers.values()))._pool
+        with pool.get_socket(all_credentials) as sock_info:
+            authset = sock_info.authset
+        cached = set(all_credentials.values())
+        self.assertEqual(len(cached), 1)
+        self.assertFalse(authset - cached)
+        self.assertFalse(cached - authset)
+
+        sock_credentials = next(iter(authset))
+        sock_cache = sock_credentials.cache
+        self.assertIsNotNone(sock_cache)
+        self.assertEqual(sock_cache.data, data)
+
+    def test_scram_threaded(self):
+
+        coll = client_context.client.db.test
+        coll.drop()
+        coll.insert_one({'_id': 1})
+
+        # The first thread to call find() will authenticate
+        coll = rs_or_single_client().db.test
+        threads = []
+        for _ in range(4):
+            threads.append(AutoAuthenticateThread(coll))
+        for thread in threads:
+            thread.start()
+        for thread in threads:
+            thread.join()
+            self.assertTrue(thread.success)
+
+class TestThreadedAuth(unittest.TestCase):
+
+    @client_context.require_auth
+    def test_db_authenticate_threaded(self):
+
+        db = client_context.client.db
+        coll = db.test
+        coll.drop()
+        coll.insert_one({'_id': 1})
+
+        client_context.create_user(
+            'db',
+            'user',
+            'pass',
+            roles=['dbOwner'])
+        self.addCleanup(db.command, 'dropUser', 'user')
+
+        db = rs_or_single_client_noauth().db
+        db.authenticate('user', 'pass')
+        # No error.
+        db.authenticate('user', 'pass')
+
+        db = rs_or_single_client_noauth().db
+        threads = []
+        for _ in range(4):
+            threads.append(DBAuthenticateThread(db, 'user', 'pass'))
+        for thread in threads:
+            thread.start()
+        for thread in threads:
+            thread.join()
+            self.assertTrue(thread.success)
+
+
 class TestAuthURIOptions(unittest.TestCase):
 
     @client_context.require_auth
