@@ -26,8 +26,7 @@ from collections import defaultdict
 
 sys.path[0:0] = [""]
 
-import bson
-
+from bson import encode
 from bson.raw_bson import RawBSONDocument
 from bson.regex import Regex
 from bson.code import Code
@@ -765,7 +764,7 @@ class TestCollection(IntegrationTest):
                    'forcing duplicate key error')
 
         document = RawBSONDocument(
-            bson.BSON.encode({'_id': ObjectId(), 'foo': 'bar'}))
+            encode({'_id': ObjectId(), 'foo': 'bar'}))
         result = db.test.insert_one(document)
         self.assertTrue(isinstance(result, InsertOneResult))
         self.assertEqual(result.inserted_id, None)
@@ -798,7 +797,7 @@ class TestCollection(IntegrationTest):
             self.assertEqual(1, db.test.count_documents({"_id": _id}))
         self.assertTrue(result.acknowledged)
 
-        docs = [RawBSONDocument(bson.BSON.encode({"_id": i + 5}))
+        docs = [RawBSONDocument(encode({"_id": i + 5}))
                 for i in range(5)]
         result = db.test.insert_many(docs)
         self.assertTrue(isinstance(result, InsertManyResult))
@@ -1336,6 +1335,12 @@ class TestCollection(IntegrationTest):
         self.assertIn('E11000 duplicate key error',
                       str(ctx.exception))
 
+        if sys.version_info[0] == 2:
+            # Test unicode(error) conversion.
+            self.assertIn('E11000 duplicate key error',
+                          unicode(ctx.exception))
+
+
     def test_wtimeout(self):
         # Ensure setting wtimeout doesn't disable write concern altogether.
         # See SERVER-12596.
@@ -1397,7 +1402,7 @@ class TestCollection(IntegrationTest):
         self.assertEqual(0, db.test.count_documents({"x": 1}))
         self.assertEqual(db.test.find_one(id1)["y"], 1)
 
-        replacement = RawBSONDocument(bson.BSON.encode({"_id": id1, "z": 1}))
+        replacement = RawBSONDocument(encode({"_id": id1, "z": 1}))
         result = db.test.replace_one({"y": 1}, replacement, True)
         self.assertTrue(isinstance(result, UpdateResult))
         self.assertEqual(1, result.matched_count)
@@ -2074,6 +2079,8 @@ class TestCollection(IntegrationTest):
         self.assertEqual(2, result.find_one({"_id": "dog"})["value"])
         self.assertEqual(1, result.find_one({"_id": "mouse"})["value"])
 
+        # Create the output database.
+        db.client.mrtestdb.mrunittests.insert_one({})
         result = db.test.map_reduce(map, reduce,
                                     out=SON([('replace', 'mrunittests'),
                                              ('db', 'mrtestdb')
@@ -2085,7 +2092,9 @@ class TestCollection(IntegrationTest):
 
         full_result = db.test.map_reduce(map, reduce,
                                          out='mrunittests', full_response=True)
-        self.assertEqual(6, full_result["counts"]["emit"])
+        self.assertEqual('mrunittests', full_result["result"])
+        if client_context.version < (4, 3):
+            self.assertEqual(6, full_result["counts"]["emit"])
 
         result = db.test.map_reduce(map, reduce, out='mrunittests', limit=2)
         self.assertEqual(2, result.find_one({"_id": "cat"})["value"])
@@ -2106,7 +2115,9 @@ class TestCollection(IntegrationTest):
 
         full_result = db.test.inline_map_reduce(map, reduce,
                                                 full_response=True)
-        self.assertEqual(6, full_result["counts"]["emit"])
+        self.assertEqual(3, len(full_result["results"]))
+        if client_context.version < (4, 3):
+            self.assertEqual(6, full_result["counts"]["emit"])
 
         with self.write_concern_collection() as coll:
             coll.map_reduce(map, reduce, 'output')
