@@ -18,12 +18,13 @@ import base64
 import copy
 import os
 import traceback
+import socket
 import sys
 import uuid
 
 sys.path[0:0] = [""]
 
-from bson import BSON, json_util
+from bson import encode, json_util
 from bson.binary import (Binary,
                          JAVA_LEGACY,
                          STANDARD,
@@ -34,15 +35,15 @@ from bson.json_util import JSONOptions
 from bson.son import SON
 
 from pymongo.cursor import CursorType
-from pymongo.errors import (ConfigurationError,
-                            EncryptionError,
-                            InvalidOperation,
-                            OperationFailure)
 from pymongo.encryption import (Algorithm,
                                 ClientEncryption)
-from pymongo.errors import ConfigurationError, DocumentTooLarge
 from pymongo.encryption_options import AutoEncryptionOpts, _HAVE_PYMONGOCRYPT
-from pymongo.message import _COMMAND_OVERHEAD
+from pymongo.errors import (BulkWriteError,
+                            ConfigurationError,
+                            EncryptionError,
+                            InvalidOperation,
+                            OperationFailure,
+                            WriteError)
 from pymongo.mongo_client import MongoClient
 from pymongo.operations import InsertOne
 from pymongo.write_concern import WriteConcern
@@ -166,7 +167,7 @@ def json_data(*paths):
 
 
 def bson_data(*paths):
-    return BSON.encode(json_data(*paths), codec_options=OPTS)
+    return encode(json_data(*paths), codec_options=OPTS)
 
 
 class TestClientSimple(EncryptionIntegrationTest):
@@ -585,8 +586,11 @@ class TestDataKeyDoubleEncryption(EncryptionIntegrationTest):
         return {'aws': AWS_CREDS, 'local': {'key': LOCAL_MASTER_KEY}}
 
     def test_data_key(self):
-        self.client.db.coll.drop()
-        vault = create_key_vault(self.client.admin.datakeys)
+        listener = OvertCommandListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        self.addCleanup(client.close)
+        client.db.coll.drop()
+        vault = create_key_vault(client.admin.datakeys)
         self.addCleanup(vault.drop)
 
         # Configure the encrypted field via the local schema_map option.
@@ -611,14 +615,17 @@ class TestDataKeyDoubleEncryption(EncryptionIntegrationTest):
         self.addCleanup(client_encrypted.close)
 
         client_encryption = ClientEncryption(
-            self.kms_providers(), 'admin.datakeys', client_context.client,
-            OPTS)
+            self.kms_providers(), 'admin.datakeys', client, OPTS)
         self.addCleanup(client_encryption.close)
 
         # Local create data key.
+        listener.reset()
         local_datakey_id = client_encryption.create_data_key(
             'local', key_alt_names=['local_altname'])
         self.assertBinaryUUID(local_datakey_id)
+        cmd = listener.results['started'][-1]
+        self.assertEqual('insert', cmd.command_name)
+        self.assertEqual({'w': 'majority'}, cmd.command.get('writeConcern'))
         docs = list(vault.find({'_id': local_datakey_id}))
         self.assertEqual(len(docs), 1)
         self.assertEqual(docs[0]['masterKey']['provider'], 'local')
@@ -642,6 +649,7 @@ class TestDataKeyDoubleEncryption(EncryptionIntegrationTest):
         self.assertEqual(local_encrypted_altname, local_encrypted)
 
         # AWS create data key.
+        listener.reset()
         master_key = {
             'region': 'us-east-1',
             'key': 'arn:aws:kms:us-east-1:579766882180:key/89fcc2c4-08b0-4bd9-'
@@ -650,6 +658,9 @@ class TestDataKeyDoubleEncryption(EncryptionIntegrationTest):
         aws_datakey_id = client_encryption.create_data_key(
             'aws', master_key=master_key, key_alt_names=['aws_altname'])
         self.assertBinaryUUID(aws_datakey_id)
+        cmd = listener.results['started'][-1]
+        self.assertEqual('insert', cmd.command_name)
+        self.assertEqual({'w': 'majority'}, cmd.command.get('writeConcern'))
         docs = list(vault.find({'_id': aws_datakey_id}))
         self.assertEqual(len(docs), 1)
         self.assertEqual(docs[0]['masterKey']['provider'], 'aws')
@@ -673,8 +684,8 @@ class TestDataKeyDoubleEncryption(EncryptionIntegrationTest):
         self.assertEqual(aws_encrypted_altname, aws_encrypted)
 
         # Explicitly encrypting an auto encrypted field.
-        msg = ('Cannot encrypt element of type binData because schema '
-               'requires that type is one of: \[ string \]')
+        msg = (r'Cannot encrypt element of type binData because schema '
+               r'requires that type is one of: \[ string \]')
         with self.assertRaisesRegex(EncryptionError, msg):
             client_encrypted.db.coll.insert_one(
                 {'encrypted_placeholder': local_encrypted})
@@ -907,6 +918,10 @@ class TestCorpus(EncryptionIntegrationTest):
         self._test_corpus(opts)
 
 
+_2_MiB = 2097152
+_16_MiB = 16777216
+
+
 class TestBsonSizeBatches(EncryptionIntegrationTest):
     """Prose tests for BSON size limits and batch splitting."""
 
@@ -944,27 +959,14 @@ class TestBsonSizeBatches(EncryptionIntegrationTest):
         super(TestBsonSizeBatches, cls).tearDownClass()
 
     def test_01_insert_succeeds_under_2MiB(self):
-        doc = {'_id': 'no_encryption_under_2mib',
-               'unencrypted': 'a' * ((2**21) - 1000)}
+        doc = {'_id': 'over_2mib_under_16mib', 'unencrypted': 'a' * _2_MiB}
         self.coll_encrypted.insert_one(doc)
 
         # Same with bulk_write.
-        doc = {'_id': 'no_encryption_under_2mib_bulk',
-               'unencrypted': 'a' * ((2**21) - 1000)}
+        doc['_id'] = 'over_2mib_under_16mib_bulk'
         self.coll_encrypted.bulk_write([InsertOne(doc)])
 
-    def test_02_insert_fails_over_2MiB(self):
-        doc = {'_id': 'no_encryption_over_2mib',
-               'unencrypted': 'a' * (2**21 + _COMMAND_OVERHEAD)}
-
-        with self.assertRaises(DocumentTooLarge):
-            self.coll_encrypted.insert_one(doc)
-        with self.assertRaises(DocumentTooLarge):
-            self.coll_encrypted.insert_many([doc])
-        with self.assertRaises(DocumentTooLarge):
-            self.coll_encrypted.bulk_write([InsertOne(doc)])
-
-    def test_03_insert_succeeds_over_2MiB_post_encryption(self):
+    def test_02_insert_succeeds_over_2MiB_post_encryption(self):
         doc = {'_id': 'encryption_exceeds_2mib',
                'unencrypted': 'a' * ((2**21) - 2000)}
         doc.update(json_data('limits', 'limits-doc.json'))
@@ -974,29 +976,134 @@ class TestBsonSizeBatches(EncryptionIntegrationTest):
         doc['_id'] = 'encryption_exceeds_2mib_bulk'
         self.coll_encrypted.bulk_write([InsertOne(doc)])
 
-    def test_04_bulk_batch_split(self):
-        doc1 = {'_id': 'no_encryption_under_2mib_1',
-                'unencrypted': 'a' * ((2**21) - 1000)}
-        doc2 = {'_id': 'no_encryption_under_2mib_2',
-                'unencrypted': 'a' * ((2**21) - 1000)}
+    def test_03_bulk_batch_split(self):
+        doc1 = {'_id': 'over_2mib_1', 'unencrypted': 'a' * _2_MiB}
+        doc2 = {'_id': 'over_2mib_2', 'unencrypted': 'a' * _2_MiB}
         self.listener.reset()
         self.coll_encrypted.bulk_write([InsertOne(doc1), InsertOne(doc2)])
         self.assertEqual(
             self.listener.started_command_names(), ['insert', 'insert'])
 
-    def test_05_bulk_batch_split(self):
+    def test_04_bulk_batch_split(self):
         limits_doc = json_data('limits', 'limits-doc.json')
         doc1 = {'_id': 'encryption_exceeds_2mib_1',
-                'unencrypted': 'a' * ((2**21) - 2000)}
+                'unencrypted': 'a' * (_2_MiB - 2000)}
         doc1.update(limits_doc)
         doc2 = {'_id': 'encryption_exceeds_2mib_2',
-                'unencrypted': 'a' * ((2**21) - 2000)}
+                'unencrypted': 'a' * (_2_MiB - 2000)}
         doc2.update(limits_doc)
         self.listener.reset()
         self.coll_encrypted.bulk_write([InsertOne(doc1), InsertOne(doc2)])
         self.assertEqual(
             self.listener.started_command_names(), ['insert', 'insert'])
 
+    def test_05_insert_succeeds_just_under_16MiB(self):
+        doc = {'_id': 'under_16mib', 'unencrypted': 'a' * (_16_MiB - 2000)}
+        self.coll_encrypted.insert_one(doc)
+
+        # Same with bulk_write.
+        doc['_id'] = 'under_16mib_bulk'
+        self.coll_encrypted.bulk_write([InsertOne(doc)])
+
+    def test_06_insert_fails_over_16MiB(self):
+        limits_doc = json_data('limits', 'limits-doc.json')
+        doc = {'_id': 'encryption_exceeds_16mib',
+               'unencrypted': 'a' * (_16_MiB - 2000)}
+        doc.update(limits_doc)
+
+        with self.assertRaisesRegex(WriteError, 'object to insert too large'):
+            self.coll_encrypted.insert_one(doc)
+
+        # Same with bulk_write.
+        doc['_id'] = 'encryption_exceeds_16mib_bulk'
+        with self.assertRaises(BulkWriteError) as ctx:
+            self.coll_encrypted.bulk_write([InsertOne(doc)])
+        err = ctx.exception.details['writeErrors'][0]
+        self.assertEqual(2, err['code'])
+        self.assertIn('object to insert too large', err['errmsg'])
+
+
+
+class TestCustomEndpoint(EncryptionIntegrationTest):
+    """Prose tests for creating data keys with a custom endpoint."""
+
+    @classmethod
+    @unittest.skipUnless(all(AWS_CREDS.values()),
+                         'AWS environment credentials are not set')
+    def setUpClass(cls):
+        super(TestCustomEndpoint, cls).setUpClass()
+        cls.client_encryption = ClientEncryption(
+            {'aws': AWS_CREDS}, 'admin.datakeys', client_context.client, OPTS)
+
+    def _test_create_data_key(self, master_key):
+        data_key_id = self.client_encryption.create_data_key(
+            'aws', master_key=master_key)
+        encrypted = self.client_encryption.encrypt(
+            'test', Algorithm.AEAD_AES_256_CBC_HMAC_SHA_512_Deterministic,
+            key_id=data_key_id)
+        self.assertEqual('test', self.client_encryption.decrypt(encrypted))
+
+    def test_02_aws_region_key(self):
+        self._test_create_data_key({
+            "region": "us-east-1",
+            "key": ("arn:aws:kms:us-east-1:579766882180:key/"
+                    "89fcc2c4-08b0-4bd9-9f25-e30687b580d0")
+        })
+
+    def test_03_aws_region_key_endpoint(self):
+        self._test_create_data_key({
+            "region": "us-east-1",
+            "key": ("arn:aws:kms:us-east-1:579766882180:key/"
+                    "89fcc2c4-08b0-4bd9-9f25-e30687b580d0"),
+            "endpoint": "kms.us-east-1.amazonaws.com"
+        })
+
+    def test_04_aws_region_key_endpoint_port(self):
+        self._test_create_data_key({
+            "region": "us-east-1",
+            "key": ("arn:aws:kms:us-east-1:579766882180:key/"
+                    "89fcc2c4-08b0-4bd9-9f25-e30687b580d0"),
+            "endpoint": "kms.us-east-1.amazonaws.com:443"
+        })
+
+    def test_05_endpoint_invalid_port(self):
+        master_key = {
+            "region": "us-east-1",
+            "key": ("arn:aws:kms:us-east-1:579766882180:key/"
+                    "89fcc2c4-08b0-4bd9-9f25-e30687b580d0"),
+            "endpoint": "kms.us-east-1.amazonaws.com:12345"
+        }
+        with self.assertRaises(EncryptionError) as ctx:
+            self.client_encryption.create_data_key(
+                'aws', master_key=master_key)
+        self.assertIsInstance(ctx.exception.cause, socket.error)
+
+    def test_05_endpoint_wrong_region(self):
+        master_key = {
+            "region": "us-east-1",
+            "key": ("arn:aws:kms:us-east-1:579766882180:key/"
+                    "89fcc2c4-08b0-4bd9-9f25-e30687b580d0"),
+            "endpoint": "kms.us-east-2.amazonaws.com"
+        }
+        # The full error should be something like:
+        # "Credential should be scoped to a valid region, not 'us-east-1'"
+        # but we only check for "us-east-1" to avoid breaking on slight
+        # changes to AWS' error message.
+        with self.assertRaisesRegex(EncryptionError, 'us-east-1'):
+            self.client_encryption.create_data_key(
+                'aws', master_key=master_key)
+
+    def test_05_endpoint_invalid_host(self):
+        master_key = {
+            "region": "us-east-1",
+            "key": ("arn:aws:kms:us-east-1:579766882180:key/"
+                    "89fcc2c4-08b0-4bd9-9f25-e30687b580d0"),
+            "endpoint": "example.com"
+        }
+        with self.assertRaisesRegex(EncryptionError, 'parse error'):
+            self.client_encryption.create_data_key(
+                'aws', master_key=master_key)
+
 
 if __name__ == "__main__":
     unittest.main()
