@@ -27,8 +27,7 @@ from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 
 from test import client_context, unittest, IntegrationTest
-from test.utils import rs_or_single_client
-
+from test.utils import rs_client, rs_or_single_client
 
 class TestSampleShellCommands(unittest.TestCase):
 
@@ -1037,82 +1036,43 @@ class TestTransactionExamples(IntegrationTest):
         self.assertIsNotNone(employee)
         self.assertEqual(employee['status'], 'Inactive')
 
-    @client_context.require_transactions
-    def test_transactions_beta(self):
-        # Transaction beta examples
-        client = self.client
-        self.addCleanup(client.drop_database, "test")
+        MongoClient = lambda _: rs_client()
+        uriString = None
 
-        db = client.test
-        shipment = db.create_collection("shipment")
-        inventory = db.create_collection("inventory")
-        inventory.insert_one({"sku": "abc123", "qty": 500})
+        # Start Transactions withTxn API Example 1
 
-        # Start Beta Transaction Example 1
-        db = client.test
-        with client.start_session() as s:
-            with s.start_transaction():
-                db.inventory.update_one({'sku': 'abc123'},
-                                        {'$inc': {'qty': -100}},
-                                        session=s)
-                db.shipment.insert_one({'sku': 'abc123', 'qty': 100},
-                                       session=s)
-        # End Beta Transaction Example 1
+        # For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
+        # uriString = 'mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl'
+        # For a sharded cluster, connect to the mongos instances; e.g.
+        # uriString = 'mongodb://mongos0.example.com:27017,mongos1.example.com:27017/'
 
-        # Beta Transaction Example 1 with explicit start, commit, and abort.
-        with client.start_session() as s:
-            s.start_transaction()
-            try:
-                db.inventory.update_one({'sku': 'abc123'},
-                                        {'$inc': {'qty': -100}},
-                                        session=s)
-                db.shipment.insert_one({'sku': 'abc123', 'qty': 100},
-                                       session=s)
-            except Exception:
-                s.abort_transaction()
-                raise
-            s.commit_transaction()
+        client = MongoClient(uriString)
+        wc_majority = WriteConcern("majority", wtimeout=1000)
 
-        # Start Beta Transaction Example 2
-        db = client.test
-        shipment = db.get_collection('shipment',
-                                     write_concern=WriteConcern(w='majority'))
+        # Prereq: Create collections. CRUD operations in transactions must be on existing collections.
+        client.get_database(
+            "mydb1", write_concern=wc_majority).foo.insert_one({'abc': 0})
+        client.get_database(
+            "mydb2", write_concern=wc_majority).bar.insert_one({'xyz': 0})
 
-        # In the following block, the following write concerns are used:
-        # the update_one and insert_one operations uses w = 1,
-        # the transaction commit/abort uses w = 'majority'.
-        with client.start_session() as s:
-            with s.start_transaction(write_concern=WriteConcern(w='majority')):
-                inventory.update_one({'sku': 'abc123'},
-                                     {'$inc': {'qty': -100}},
-                                     session=s)
-                shipment.insert_one({'sku': 'abc123', 'qty': 100}, session=s)
-        # End Beta Transaction Example 2
+        # Step 1: Define the callback that specifies the sequence of operations to perform inside the transactions.
+        def callback(session):
+            collection_one = session.client.mydb1.foo
+            collection_two = session.client.mydb2.bar
 
-        # Start Beta Transaction Example 3
-        def run_transaction(session, txn_callback):
-            with session.start_transaction():
-                txn_callback(session)
-
-        def run_transaction_with_retry(session, txn_callback):
-            try:
-                run_transaction(session, txn_callback)
-            except (OperationFailure, ConnectionFailure) as exc:
-                if exc.has_error_label("TransientTransactionError"):
-                    # Retry the entire transaction on temporary transaction
-                    # failures.
-                    run_transaction(session, txn_callback)
-                else:
-                    raise
-
-        def shipment_transaction(session):
-            inventory.update_one({'sku': 'abc123'}, {'$inc': {'qty': -100}},
-                                 session=session)
-            shipment.insert_one({'sku': 'abc123', 'qty': 100}, session=session)
+            # Important:: You must pass the session to the operations.
+            collection_one.insert_one({'abc': 1}, session=session)
+            collection_two.insert_one({'xyz': 999}, session=session)
 
+        # Step 2: Start a client session.
         with client.start_session() as session:
-            run_transaction_with_retry(session, shipment_transaction)
-        # End Beta Transaction Example 3
+            # Step 3: Use with_transaction to start a transaction, execute the callback, and commit (or abort on error).
+            session.with_transaction(
+                callback, read_concern=ReadConcern('local'),
+                write_concern=wc_majority,
+                read_preference=ReadPreference.PRIMARY)
+
+        # End Transactions withTxn API Example 1
 
 
 class TestCausalConsistencyExamples(IntegrationTest):
