@@ -1,4 +1,4 @@
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,12 +20,6 @@ import re
 import sys
 import uuid
 
-try:
-    import simplejson
-    HAS_SIMPLE_JSON = True
-except ImportError:
-    HAS_SIMPLE_JSON = False
-
 sys.path[0:0] = [""]
 
 from pymongo.errors import ConfigurationError
@@ -33,8 +27,9 @@ from pymongo.errors import ConfigurationError
 from bson import json_util, EPOCH_AWARE, EPOCH_NAIVE, SON
 from bson.json_util import (DatetimeRepresentation,
                             STRICT_JSON_OPTIONS)
-from bson.binary import (Binary, MD5_SUBTYPE, USER_DEFINED_SUBTYPE,
-                         JAVA_LEGACY, CSHARP_LEGACY, STANDARD)
+from bson.binary import (ALL_UUID_REPRESENTATIONS, Binary, MD5_SUBTYPE,
+                         USER_DEFINED_SUBTYPE, JAVA_LEGACY, CSHARP_LEGACY,
+                         STANDARD)
 from bson.code import Code
 from bson.dbref import DBRef
 from bson.int64 import Int64
@@ -48,7 +43,6 @@ from bson.tz_util import FixedOffset, utc
 from test import unittest, IntegrationTest
 
 PY3 = sys.version_info[0] == 3
-PY26 = sys.version_info[:2] == (2, 6)
 
 
 class TestJsonUtil(unittest.TestCase):
@@ -93,9 +87,13 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
         jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000Z"}}'
         self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
+        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00Z"}}'
+        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
         # No explicit offset
         jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000"}}'
         self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
+        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00"}}'
+        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
         jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000"}}'
         self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
         # Localtime behind UTC
@@ -248,10 +246,9 @@ class TestJsonUtil(unittest.TestCase):
     def test_timestamp(self):
         dct = {"ts": Timestamp(4, 13)}
         res = json_util.dumps(dct, default=json_util.default)
-        self.assertEqual('{"ts": {"$timestamp": {"t": 4, "i": 13}}}', res)
-
         rtdct = json_util.loads(res)
         self.assertEqual(dct, rtdct)
+        self.assertEqual('{"ts": {"$timestamp": {"t": 4, "i": 13}}}', res)
 
     def test_uuid(self):
         doc = {'uuid': uuid.UUID('f47ac10b-58cc-4372-a567-0e02b2c3d479')}
@@ -260,23 +257,35 @@ class TestJsonUtil(unittest.TestCase):
             '{"uuid": {"$uuid": "f47ac10b58cc4372a5670e02b2c3d479"}}',
             json_util.dumps(doc))
         self.assertEqual(
-            '{"uuid": {"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}',
-            json_util.dumps(doc, json_options=json_util.STRICT_JSON_OPTIONS))
+            '{"uuid": '
+            '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}',
+            json_util.dumps(
+                doc, json_options=json_util.STRICT_JSON_OPTIONS))
         self.assertEqual(
-            '{"uuid": {"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
-            json_util.dumps(doc, json_options=json_util.JSONOptions(
-                strict_uuid=True, uuid_representation=STANDARD)))
-        self.assertEqual(doc, json_util.loads(
-            '{"uuid": {"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}'))
-        self.assertEqual(doc, json_util.loads(
-            '{"uuid": {"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}'))
-        self.round_trip(doc, json_options=json_util.JSONOptions(
-            strict_uuid=True, uuid_representation=JAVA_LEGACY))
-        self.round_trip(doc, json_options=json_util.JSONOptions(
-            strict_uuid=True, uuid_representation=CSHARP_LEGACY))
+            '{"uuid": '
+            '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
+            json_util.dumps(
+                doc, json_options=json_util.JSONOptions(
+                    strict_uuid=True, uuid_representation=STANDARD)))
+        self.assertEqual(
+            doc, json_util.loads(
+                '{"uuid": '
+                '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}'))
+        for uuid_representation in ALL_UUID_REPRESENTATIONS:
+            options = json_util.JSONOptions(
+                strict_uuid=True, uuid_representation=uuid_representation)
+            self.round_trip(doc, json_options=options)
+            # Ignore UUID representation when decoding BSON binary subtype 4.
+            self.assertEqual(doc, json_util.loads(
+                '{"uuid": '
+                '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
+                json_options=options))
 
     def test_binary(self):
-        bin_type_dict = {"bin": Binary(b"\x00\x01\x02\x03\x04")}
+        if PY3:
+            bin_type_dict = {"bin": b"\x00\x01\x02\x03\x04"}
+        else:
+            bin_type_dict = {"bin": Binary(b"\x00\x01\x02\x03\x04")}
         md5_type_dict = {
             "md5": Binary(b' n7\x18\xaf\t/\xd1\xd1/\x80\xca\xe7q\xcc\xac',
                 MD5_SUBTYPE)}
@@ -286,12 +295,19 @@ class TestJsonUtil(unittest.TestCase):
         self.round_trip(md5_type_dict)
         self.round_trip(custom_type_dict)
 
+        # Binary with subtype 0 is decoded into bytes in Python 3.
+        bin = json_util.loads(
+            '{"bin": {"$binary": "AAECAwQ=", "$type": "00"}}')['bin']
+        if PY3:
+            self.assertEqual(type(bin), bytes)
+        else:
+            self.assertEqual(type(bin), Binary)
+
         # PYTHON-443 ensure old type formats are supported
         json_bin_dump = json_util.dumps(bin_type_dict)
         self.assertTrue('"$type": "00"' in json_bin_dump)
         self.assertEqual(bin_type_dict,
             json_util.loads('{"bin": {"$type": 0, "$binary": "AAECAwQ="}}'))
-
         json_bin_dump = json_util.dumps(md5_type_dict)
         # Check order.
         self.assertEqual(
@@ -352,13 +368,9 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual({"foo": "bar"}, json_util.loads(
             '{"foo": "bar"}',
             json_options=json_util.JSONOptions(document_class=dict)))
-        if PY26 and not HAS_SIMPLE_JSON:
-            self.assertRaises(
-                ConfigurationError, json_util.JSONOptions, document_class=SON)
-        else:
-            self.assertEqual(SON([("foo", "bar"), ("b", 1)]), json_util.loads(
-                '{"foo": "bar", "b": 1}',
-                json_options=json_util.JSONOptions(document_class=SON)))
+        self.assertEqual(SON([("foo", "bar"), ("b", 1)]), json_util.loads(
+            '{"foo": "bar", "b": 1}',
+            json_options=json_util.JSONOptions(document_class=SON)))
 
 
 class TestJsonUtilRoundtrip(IntegrationTest):
@@ -370,7 +382,7 @@ class TestJsonUtilRoundtrip(IntegrationTest):
             {'foo': [1, 2]},
             {'bar': {'hello': 'world'}},
             {'code': Code("function x() { return 1; }")},
-            {'bin': Binary(b"\x00\x01\x02\x03\x04")},
+            {'bin': Binary(b"\x00\x01\x02\x03\x04", USER_DEFINED_SUBTYPE)},
             {'dbref': {'_ref': DBRef('simple',
                                ObjectId('509b8db456c02c5ab7e63c34'))}}
         ]
