@@ -26,7 +26,6 @@ sys.path[0:0] = [""]
 from bson.binary import PYTHON_LEGACY, STANDARD
 from bson.code import Code
 from bson.codec_options import CodecOptions
-from bson.dbref import DBRef
 from bson.objectid import ObjectId
 from bson.py3compat import string_type
 from bson.son import SON
@@ -54,8 +53,7 @@ from test.test_client import IntegrationTest
 from test.test_bulk import BulkTestBase, BulkAuthorizationTestBase
 from test.utils import (DeprecationFilter,
                         joinall,
-                        oid_generated_on_client,
-                        remove_all_users,
+                        oid_generated_on_process,
                         rs_or_single_client,
                         rs_or_single_client_noauth,
                         single_client,
@@ -915,6 +913,7 @@ class TestLegacy(IntegrationTest):
                          c.find_and_modify({'_id': 1}, {'$inc': {'i': 1}},
                                            new=True))
 
+    @client_context.require_version_max(4, 1, 0, -1)
     def test_group(self):
         db = self.db
         db.drop_collection("test")
@@ -979,6 +978,7 @@ class TestLegacy(IntegrationTest):
         self.assertRaises(OperationFailure, db.test.group,
                           [], {}, {}, "5 ++ 5")
 
+    @client_context.require_version_max(4, 1, 0, -1)
     def test_group_with_scope(self):
         db = self.db
         db.drop_collection("test")
@@ -1010,6 +1010,7 @@ class TestLegacy(IntegrationTest):
             [], {}, {"count": 0},
             Code(reduce_function, {"inc_value": 0.5}))[0]['count'])
 
+    @client_context.require_version_max(4, 1, 0, -1)
     def test_group_uuid_representation(self):
         db = self.db
         coll = db.uuid
@@ -1039,7 +1040,12 @@ class TestLegacy(IntegrationTest):
         collection.save({"i": 1})
 
         collection.update({"i": 1}, {"$set": {"i": 2}}, w=0)
-        self.assertTrue(db.last_status()["updatedExisting"])
+        # updatedExisting is always false on mongos after an OP_MSG
+        # unacknowledged write.
+        if not (client_context.version >= (3, 6) and client_context.is_mongos):
+            self.assertTrue(db.last_status()["updatedExisting"])
+        wait_until(lambda: collection.find_one({"i": 2}),
+                   "found updated w=0 doc")
 
         collection.update({"i": 1}, {"$set": {"i": 500}}, w=0)
         self.assertFalse(db.last_status()["updatedExisting"])
@@ -1257,6 +1263,7 @@ class TestLegacy(IntegrationTest):
         # Clean up indexes for later tests
         db.test.drop_indexes()
 
+    @client_context.require_version_max(4, 1)  # PYTHON-1734
     def test_ensure_index_threaded(self):
         coll = self.db.threaded_index_creation
         index_docs = []
@@ -1317,6 +1324,7 @@ class TestLegacy(IntegrationTest):
         finally:
             coll.drop()
 
+    @client_context.require_version_max(4, 1)  # PYTHON-1734
     def test_ensure_unique_index_threaded(self):
         coll = self.db.test_unique_threaded
         coll.drop()
@@ -1371,9 +1379,12 @@ class TestLegacy(IntegrationTest):
         wait_until(raises_cursor_not_found, 'close cursor')
 
     def test_kill_cursors_with_tuple(self):
-        if (client_context.version[:2] == (3, 6)
-                and client_context.auth_enabled):
-            raise SkipTest("SERVER-33553")
+        # Some evergreen distros (Debian 7.1) still test against 3.6.5 where
+        # OP_KILL_CURSORS does not work.
+        if (client_context.is_mongos and client_context.auth_enabled and
+                (3, 6, 0) <= client_context.version < (3, 6, 6)):
+            raise SkipTest("SERVER-33553 This server version does not support "
+                           "OP_KILL_CURSORS")
 
         coll = self.client.pymongo_test.test
         coll.drop()
@@ -1399,26 +1410,6 @@ class TestLegacy(IntegrationTest):
 
         wait_until(raises_cursor_not_found, 'close cursor')
 
-    def test_get_default_database(self):
-        c = rs_or_single_client("mongodb://%s:%d/foo" % (client_context.host,
-                                                         client_context.port),
-                                connect=False)
-        self.assertEqual(Database(c, 'foo'), c.get_default_database())
-
-    def test_get_default_database_error(self):
-        # URI with no database.
-        c = rs_or_single_client("mongodb://%s:%d/" % (client_context.host,
-                                                      client_context.port),
-                                connect=False)
-        self.assertRaises(ConfigurationError, c.get_default_database)
-
-    def test_get_default_database_with_authsource(self):
-        # Ensure we distinguish database name from authSource.
-        uri = "mongodb://%s:%d/foo?authSource=src" % (
-            client_context.host, client_context.port)
-        c = rs_or_single_client(uri, connect=False)
-        self.assertEqual(Database(c, 'foo'), c.get_default_database())
-
 
 class TestLegacyBulk(BulkTestBase):
 
@@ -1506,7 +1497,7 @@ class TestLegacyBulk(BulkTestBase):
 
         self.assertEqual(1, self.coll.count())
         doc = self.coll.find_one()
-        self.assertTrue(oid_generated_on_client(doc['_id']))
+        self.assertTrue(oid_generated_on_process(doc['_id']))
 
         bulk = self.coll.initialize_unordered_bulk_op()
         bulk.insert({})
