@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,19 +14,18 @@
 
 """Test the pymongo module itself."""
 
-import unittest
 import sys
 sys.path[0:0] = [""]
 
 import pymongo
-from test import host, port
+from test import unittest
+
 
 class TestPyMongo(unittest.TestCase):
     def test_mongo_client_alias(self):
         # Testing that pymongo module imports mongo_client.MongoClient
-        c = pymongo.MongoClient(host, port)
-        self.assertEqual(c.host, host)
-        self.assertEqual(c.port, port)
+        self.assertEqual(pymongo.MongoClient,
+                         pymongo.mongo_client.MongoClient)
 
 
 if __name__ == "__main__":
