@@ -318,9 +318,18 @@ class ReadPrefTester(MongoClient):
         super(ReadPrefTester, self).__init__(*args, **client_options)
 
     @contextlib.contextmanager
-    def _socket_for_reads(self, read_preference):
+    def _socket_for_reads(self, read_preference, session):
         context = super(ReadPrefTester, self)._socket_for_reads(
-            read_preference)
+            read_preference, session)
+        with context as (sock_info, slave_ok):
+            self.record_a_read(sock_info.address)
+            yield sock_info, slave_ok
+
+    @contextlib.contextmanager
+    def _slaveok_for_server(self, read_preference, server, session,
+                            exhaust=False):
+        context = super(ReadPrefTester, self)._slaveok_for_server(
+            read_preference, server, session, exhaust=exhaust)
         with context as (sock_info, slave_ok):
             self.record_a_read(sock_info.address)
             yield sock_info, slave_ok
@@ -459,6 +468,11 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
                                'aggregate',
                                [{'$project': {'_id': 1}}])
 
+    def test_aggregate_write(self):
+        self._test_coll_helper(False, self.c.pymongo_test.test,
+                               'aggregate',
+                               [{'$project': {'_id': 1}}, {'$out': "agg_write_test"}])
+
 
 class TestMovingAverage(unittest.TestCase):
     def test_moving_average(self):
