@@ -14,7 +14,7 @@
 
 """Test retryable writes."""
 
-import json
+import copy
 import os
 import sys
 
@@ -35,141 +35,62 @@ from pymongo.operations import (InsertOne,
                                 ReplaceOne,
                                 UpdateMany,
                                 UpdateOne)
-from pymongo.results import BulkWriteResult
 from pymongo.write_concern import WriteConcern
 
 from test import unittest, client_context, IntegrationTest, SkipTest, client_knobs
+from test.test_crud_v1 import check_result as crud_v1_check_result
 from test.utils import (rs_or_single_client,
                         DeprecationFilter,
-                        OvertCommandListener)
-from test.test_crud import check_result, run_operation
+                        OvertCommandListener,
+                        TestCreator)
+from test.utils_spec_runner import SpecRunner
 
 # Location of JSON test specifications.
 _TEST_PATH = os.path.join(
     os.path.dirname(os.path.realpath(__file__)), 'retryable_writes')
 
 
-class TestAllScenarios(IntegrationTest):
+class TestAllScenarios(SpecRunner):
 
-    @classmethod
-    @client_context.require_version_min(3, 5)
-    @client_context.require_replica_set
-    @client_context.require_test_commands
-    def setUpClass(cls):
-        super(TestAllScenarios, cls).setUpClass()
-        # Speed up the tests by decreasing the heartbeat frequency.
-        cls.knobs = client_knobs(heartbeat_frequency=0.1,
-                                 min_heartbeat_interval=0.1)
-        cls.knobs.enable()
+    def get_object_name(self, op):
+        return op.get('object', 'collection')
 
-    @classmethod
-    def tearDownClass(cls):
-        cls.knobs.disable()
-        super(TestAllScenarios, cls).tearDownClass()
+    def get_scenario_db_name(self, scenario_def):
+        return scenario_def.get('database_name', 'pymongo_test')
 
-    def tearDown(self):
-        client_context.client.admin.command(SON([
-            ('configureFailPoint', 'onPrimaryTransactionalWrite'),
-            ('mode', 'off')]))
+    def get_scenario_coll_name(self, scenario_def):
+        return scenario_def.get('collection_name', 'test')
 
-    def set_fail_point(self, command_args):
-        cmd = SON([('configureFailPoint', 'onPrimaryTransactionalWrite')])
-        cmd.update(command_args)
-        client_context.client.admin.command(cmd)
-
-
-def create_test(scenario_def, test):
-    def run_scenario(self):
-        # Load data.
-        assert scenario_def['data'], "tests must have non-empty data"
-        client_context.client.pymongo_test.test.drop()
-        client_context.client.pymongo_test.test.insert_many(scenario_def['data'])
-
-        # Set the failPoint
-        self.set_fail_point(test['failPoint'])
-        self.addCleanup(self.set_fail_point, {
-            'configureFailPoint': test['failPoint']['configureFailPoint'],
-            'mode': 'off'})
-
-        test_outcome = test['outcome']
-        should_fail = test_outcome.get('error')
+    def run_test_ops(self, sessions, collection, test):
+        outcome = test['outcome']
+        should_fail = outcome.get('error')
         result = None
         error = None
-
-        db = rs_or_single_client(**test['clientOptions']).pymongo_test
-        # Close the client explicitly to avoid having too many threads open.
-        self.addCleanup(db.client.close)
         try:
-            result = run_operation(db.test, test)
+            result = self.run_operation(
+                sessions, collection, test['operation'])
         except (ConnectionFailure, OperationFailure) as exc:
             error = exc
-
         if should_fail:
             self.assertIsNotNone(error, 'should have raised an error')
         else:
             self.assertIsNone(error)
+            crud_v1_check_result(self, outcome['result'], result)
 
-        # Assert final state is expected.
-        expected_c = test_outcome.get('collection')
-        if expected_c is not None:
-            expected_name = expected_c.get('name')
-            if expected_name is not None:
-                db_coll = db[expected_name]
-            else:
-                db_coll = db.test
-            self.assertEqual(list(db_coll.find()), expected_c['data'])
-        expected_result = test_outcome.get('result')
-        # We can't test the expected result when the test should fail because
-        # the BulkWriteResult is not reported when raising a network error.
-        if not should_fail:
-            self.assertTrue(check_result(expected_result, result),
-                            "%r != %r" % (expected_result, result))
+
+def create_test(scenario_def, test, name):
+    @client_context.require_test_commands
+    @client_context.require_no_mmap
+    def run_scenario(self):
+        self.run_scenario(scenario_def, test)
 
     return run_scenario
 
-
-def create_tests():
-    for dirpath, _, filenames in os.walk(_TEST_PATH):
-        dirname = os.path.split(dirpath)[-1]
-
-        for filename in filenames:
-            with open(os.path.join(dirpath, filename)) as scenario_stream:
-                scenario_def = json.load(scenario_stream)
-
-            test_type = os.path.splitext(filename)[0]
-            min_ver, max_ver = None, None
-            if 'minServerVersion' in scenario_def:
-                min_ver = tuple(
-                    int(elt) for
-                    elt in scenario_def['minServerVersion'].split('.'))
-            if 'maxServerVersion' in scenario_def:
-                max_ver = tuple(
-                    int(elt) for
-                    elt in scenario_def['maxServerVersion'].split('.'))
-
-            # Construct test from scenario.
-            for test in scenario_def['tests']:
-                new_test = create_test(scenario_def, test)
-                if min_ver is not None:
-                    new_test = client_context.require_version_min(*min_ver)(
-                        new_test)
-                if max_ver is not None:
-                    new_test = client_context.require_version_max(*max_ver)(
-                        new_test)
-
-                test_name = 'test_%s_%s_%s' % (
-                    dirname,
-                    test_type,
-                    str(test['description'].replace(" ", "_")))
-
-                new_test.__name__ = test_name
-                setattr(TestAllScenarios, new_test.__name__, new_test)
+test_creator = TestCreator(create_test, TestAllScenarios, _TEST_PATH)
+test_creator.create_tests()
 
 
-create_tests()
-
-
-def retryable_single_statement_ops(coll):
+def _retryable_single_statement_ops(coll):
     return [
         (coll.bulk_write, [[InsertOne({}), InsertOne({})]], {}),
         (coll.bulk_write, [[InsertOne({}),
@@ -188,6 +109,11 @@ def retryable_single_statement_ops(coll):
         (coll.find_one_and_replace, [{}, {'a': 3}], {}),
         (coll.find_one_and_update, [{}, {'$set': {'a': 1}}], {}),
         (coll.find_one_and_delete, [{}, {}], {}),
+    ]
+
+
+def retryable_single_statement_ops(coll):
+    return _retryable_single_statement_ops(coll) + [
         # Deprecated methods.
         # Insert with single or multiple documents.
         (coll.insert, [{}], {}),
@@ -251,9 +177,41 @@ class IgnoreDeprecationsTest(IntegrationTest):
         super(IgnoreDeprecationsTest, cls).tearDownClass()
 
 
+class TestRetryableWritesMMAPv1(IgnoreDeprecationsTest):
+
+    @classmethod
+    def setUpClass(cls):
+        super(TestRetryableWritesMMAPv1, cls).setUpClass()
+        # Speed up the tests by decreasing the heartbeat frequency.
+        cls.knobs = client_knobs(heartbeat_frequency=0.1,
+                                 min_heartbeat_interval=0.1)
+        cls.knobs.enable()
+        cls.client = rs_or_single_client(retryWrites=True)
+        cls.db = cls.client.pymongo_test
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.knobs.disable()
+
+    @client_context.require_version_min(3, 5)
+    @client_context.require_no_standalone
+    def test_actionable_error_message(self):
+        if client_context.storage_engine != 'mmapv1':
+            raise SkipTest('This cluster is not running MMAPv1')
+
+        expected_msg = ("This MongoDB deployment does not support retryable "
+                        "writes. Please add retryWrites=false to your "
+                        "connection string.")
+        for method, args, kwargs in retryable_single_statement_ops(
+                self.db.retryable_write_test):
+            with self.assertRaisesRegex(OperationFailure, expected_msg):
+                method(*args, **kwargs)
+
+
 class TestRetryableWrites(IgnoreDeprecationsTest):
 
     @classmethod
+    @client_context.require_no_mmap
     def setUpClass(cls):
         super(TestRetryableWrites, cls).setUpClass()
         # Speed up the tests by decreasing the heartbeat frequency.
@@ -500,5 +458,47 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
         self.assertEqual(coll.find_one(projection={'_id': True}), {'_id': 1})
 
 
+# TODO: Make this a real integration test where we stepdown the primary.
+class TestRetryableWritesTxnNumber(IgnoreDeprecationsTest):
+    @client_context.require_version_min(3, 6)
+    @client_context.require_replica_set
+    @client_context.require_no_mmap
+    def test_increment_transaction_id_without_sending_command(self):
+        """Test that the txnNumber field is properly incremented, even when
+        the first attempt fails before sending the command.
+        """
+        listener = OvertCommandListener()
+        client = rs_or_single_client(
+            retryWrites=True, event_listeners=[listener])
+        topology = client._topology
+        select_server = topology.select_server
+
+        def raise_connection_err_select_server(*args, **kwargs):
+            # Raise ConnectionFailure on the first attempt and perform
+            # normal selection on the retry attempt.
+            topology.select_server = select_server
+            raise ConnectionFailure('Connection refused')
+
+        for method, args, kwargs in _retryable_single_statement_ops(
+                client.db.retryable_write_test):
+            listener.results.clear()
+            topology.select_server = raise_connection_err_select_server
+            with client.start_session() as session:
+                kwargs = copy.deepcopy(kwargs)
+                kwargs['session'] = session
+                msg = '%s(*%r, **%r)' % (method.__name__, args, kwargs)
+                initial_txn_id = session._server_session.transaction_id
+
+                # Each operation should fail on the first attempt and succeed
+                # on the second.
+                method(*args, **kwargs)
+                self.assertEqual(len(listener.results['started']), 1, msg)
+                retry_cmd = listener.results['started'][0].command
+                sent_txn_id = retry_cmd['txnNumber']
+                final_txn_id = session._server_session.transaction_id
+                self.assertEqual(Int64(initial_txn_id + 1), sent_txn_id, msg)
+                self.assertEqual(sent_txn_id, final_txn_id, msg)
+
+
 if __name__ == '__main__':
     unittest.main()
