@@ -20,13 +20,12 @@ import warnings
 
 sys.path[0:0] = [""]
 
-from pymongo.uri_parser import (_partition,
-                                _rpartition,
-                                parse_userinfo,
+from pymongo.uri_parser import (parse_userinfo,
                                 split_hosts,
                                 split_options,
                                 parse_uri)
 from pymongo.errors import ConfigurationError, InvalidURI
+from pymongo.ssl_support import ssl
 from pymongo import ReadPreference
 from bson.binary import JAVA_LEGACY
 from bson.py3compat import string_type, _unicode
@@ -35,14 +34,6 @@ from test import unittest
 
 class TestURI(unittest.TestCase):
 
-    def test_partition(self):
-        self.assertEqual(('foo', ':', 'bar'), _partition('foo:bar', ':'))
-        self.assertEqual(('foobar', '', ''), _partition('foobar', ':'))
-
-    def test_rpartition(self):
-        self.assertEqual(('fo:o:', ':', 'bar'), _rpartition('fo:o::bar', ':'))
-        self.assertEqual(('', '', 'foobar'), _rpartition('foobar', ':'))
-
     def test_validate_userinfo(self):
         self.assertRaises(InvalidURI, parse_userinfo,
                           'foo@')
@@ -123,16 +114,10 @@ class TestURI(unittest.TestCase):
                               'connect=foo', warn=True)
             self.assertRaises(Warning, split_options,
                               'ssl_match_hostname=foo', warn=True)
-
-            # On most platforms float('inf') and float('-inf') represent
-            # +/- infinity, although on Python 2.4 and 2.5 on Windows those
-            # expressions are invalid
-            if not (sys.platform == "win32" and sys.version_info <= (2, 5)):
-                self.assertRaises(Warning, split_options,
-                                  'connectTimeoutMS=inf', warn=True)
-                self.assertRaises(Warning, split_options,
-                                  'connectTimeoutMS=-inf', warn=True)
-
+            self.assertRaises(Warning, split_options,
+                              'connectTimeoutMS=inf', warn=True)
+            self.assertRaises(Warning, split_options,
+                              'connectTimeoutMS=-inf', warn=True)
             self.assertRaises(Warning, split_options, 'wtimeoutms=foo',
                               warn=True)
             self.assertRaises(Warning, split_options, 'wtimeoutms=5.5',
@@ -158,11 +143,8 @@ class TestURI(unittest.TestCase):
         self.assertRaises(ValueError, split_options, 'ssl=foo')
         self.assertRaises(ValueError, split_options, 'connect=foo')
         self.assertRaises(ValueError, split_options, 'ssl_match_hostname=foo')
-        if not (sys.platform == "win32" and sys.version_info <= (2, 5)):
-            self.assertRaises(ValueError, split_options,
-                              'connectTimeoutMS=inf')
-            self.assertRaises(ValueError, split_options,
-                              'connectTimeoutMS=-inf')
+        self.assertRaises(ValueError, split_options, 'connectTimeoutMS=inf')
+        self.assertRaises(ValueError, split_options, 'connectTimeoutMS=-inf')
         self.assertRaises(ValueError, split_options, 'wtimeoutms=foo')
         self.assertRaises(ValueError, split_options, 'wtimeoutms=5.5')
         self.assertRaises(ValueError, split_options, 'fsync=foo')
@@ -211,7 +193,8 @@ class TestURI(unittest.TestCase):
             'password': None,
             'database': None,
             'collection': None,
-            'options': {}
+            'options': {},
+            'fqdn': None
         }
 
         res = copy.deepcopy(orig)
@@ -452,14 +435,6 @@ class TestURI(unittest.TestCase):
                           "mongodb://user%40domain.com:password"
                           "@localhost/foo?uuidrepresentation=notAnOption")
 
-    def test_parse_uri_unicode(self):
-        # Ensure parsing a unicode returns option names that can be passed
-        # as kwargs. In Python 2.4, keyword argument names must be ASCII.
-        # In all Pythons, str is the type of valid keyword arg names.
-        res = parse_uri(_unicode("mongodb://localhost/?fsync=true"))
-        for key in res['options']:
-            self.assertTrue(isinstance(key, str))
-
     def test_parse_ssl_paths(self):
         # Turn off "validate" since these paths don't exist on filesystem.
         self.assertEqual(
@@ -468,7 +443,8 @@ class TestURI(unittest.TestCase):
              'nodelist': [('/MongoDB.sock', None)],
              'options': {'ssl_certfile': '/a/b'},
              'password': 'foo/bar',
-             'username': 'jesse'},
+             'username': 'jesse',
+             'fqdn': None},
             parse_uri(
                 'mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?ssl_certfile=/a/b',
                 validate=False))
@@ -479,11 +455,45 @@ class TestURI(unittest.TestCase):
              'nodelist': [('/MongoDB.sock', None)],
              'options': {'ssl_certfile': 'a/b'},
              'password': 'foo/bar',
-             'username': 'jesse'},
+             'username': 'jesse',
+             'fqdn': None},
             parse_uri(
                 'mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?ssl_certfile=a/b',
                 validate=False))
 
+    def test_tlsinsecure_simple(self):
+        # check that tlsInsecure is expanded correctly.
+        uri = "mongodb://example.com/?tlsInsecure=true"
+        res = {
+            "ssl_match_hostname": False, "ssl_cert_reqs": ssl.CERT_NONE,
+            "tlsinsecure": True}
+        self.assertEqual(res, parse_uri(uri)["options"])
+
+    def test_tlsinsecure_legacy_conflict(self):
+        # must not allow use of tlsinsecure alongside legacy TLS options.
+        # same check for modern TLS options is performed in the spec-tests.
+        uri = "mongodb://srv.com/?tlsInsecure=true&ssl_match_hostname=true"
+        with self.assertRaises(InvalidURI):
+            parse_uri(uri, validate=False, warn=False, normalize=False)
+
+    def test_normalize_options(self):
+        # check that options are converted to their internal names correctly.
+        uri = ("mongodb://example.com/?tls=true&appname=myapp&maxPoolSize=10&"
+               "fsync=true&wtimeout=10")
+        res = {
+            "ssl": True, "appname": "myapp", "maxpoolsize": 10,
+            "fsync": True, "wtimeoutms": 10}
+        self.assertEqual(res, parse_uri(uri)["options"])
+
+    def test_waitQueueMultiple_deprecated(self):
+        uri = "mongodb://example.com/?waitQueueMultiple=5"
+        with warnings.catch_warnings(record=True) as ctx:
+            warnings.simplefilter('always')
+            parse_uri(uri)
+
+        self.assertEqual(len(ctx), 1)
+        self.assertTrue(issubclass(ctx[0].category, DeprecationWarning))
+
 
 if __name__ == "__main__":
     unittest.main()
