@@ -15,10 +15,11 @@
 """Utilities for testing pymongo
 """
 
+import collections
 import contextlib
 import functools
 import os
-import struct
+import re
 import sys
 import threading
 import time
@@ -27,14 +28,23 @@ import warnings
 from collections import defaultdict
 from functools import partial
 
-from pymongo import MongoClient, monitoring
-from pymongo.errors import AutoReconnect, OperationFailure
-from pymongo.monitoring import _SENSITIVE_COMMANDS
+from bson import json_util, py3compat
+from bson.objectid import ObjectId
+
+from pymongo import (MongoClient,
+                     monitoring, read_preferences)
+from pymongo.errors import ConfigurationError, OperationFailure
+from pymongo.monitoring import _SENSITIVE_COMMANDS, ConnectionPoolListener
+from pymongo.pool import PoolOptions
+from pymongo.read_concern import ReadConcern
+from pymongo.read_preferences import ReadPreference
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
 from pymongo.write_concern import WriteConcern
-from test import (client_context, db_user, db_pwd)
 
+from test import (client_context,
+                  db_user,
+                  db_pwd)
 
 IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=1000)
 
@@ -58,6 +68,51 @@ class WhiteListEventListener(monitoring.CommandListener):
             self.results['failed'].append(event)
 
 
+class CMAPListener(ConnectionPoolListener):
+    def __init__(self):
+        self.events = []
+
+    def reset(self):
+        self.events = []
+
+    def add_event(self, event):
+        self.events.append(event)
+
+    def event_count(self, event_type):
+        return len([event for event in self.events[:]
+                    if isinstance(event, event_type)])
+
+    def connection_created(self, event):
+        self.add_event(event)
+
+    def connection_ready(self, event):
+        self.add_event(event)
+
+    def connection_closed(self, event):
+        self.add_event(event)
+
+    def connection_check_out_started(self, event):
+        self.add_event(event)
+
+    def connection_check_out_failed(self, event):
+        self.add_event(event)
+
+    def connection_checked_out(self, event):
+        self.add_event(event)
+
+    def connection_checked_in(self, event):
+        self.add_event(event)
+
+    def pool_created(self, event):
+        self.add_event(event)
+
+    def pool_cleared(self, event):
+        self.add_event(event)
+
+    def pool_closed(self, event):
+        self.add_event(event)
+
+
 class EventListener(monitoring.CommandListener):
 
     def __init__(self):
@@ -72,6 +127,14 @@ class EventListener(monitoring.CommandListener):
     def failed(self, event):
         self.results['failed'].append(event)
 
+    def started_command_names(self):
+        """Return list of command names started."""
+        return [event.command_name for event in self.results['started']]
+
+    def reset(self):
+        """Reset the state of this listener."""
+        self.results.clear()
+
 
 class OvertCommandListener(EventListener):
     """A CommandListener that ignores sensitive commands."""
@@ -121,13 +184,230 @@ class HeartbeatEventListener(monitoring.ServerHeartbeatListener):
         self.results.append(event)
 
 
-def _connection_string(h, p, authenticate):
+class MockSocketInfo(object):
+    def close(self):
+        pass
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
+
+class MockPool(object):
+    def __init__(self, *args, **kwargs):
+        self.pool_id = 0
+        self._lock = threading.Lock()
+        self.opts = PoolOptions()
+
+    def get_socket(self, all_credentials):
+        return MockSocketInfo()
+
+    def return_socket(self, *args, **kwargs):
+        pass
+
+    def _reset(self):
+        with self._lock:
+            self.pool_id += 1
+
+    def reset(self):
+        self._reset()
+
+    def close(self):
+        self._reset()
+
+    def update_is_writable(self, is_writable):
+        pass
+
+    def remove_stale_sockets(self, reference_pool_id):
+        pass
+
+
+class ScenarioDict(dict):
+    """Dict that returns {} for any unknown key, recursively."""
+    def __init__(self, data):
+        def convert(v):
+            if isinstance(v, collections.Mapping):
+                return ScenarioDict(v)
+            if isinstance(v, (py3compat.string_type, bytes)):
+                return v
+            if isinstance(v, collections.Sequence):
+                return [convert(item) for item in v]
+            return v
+
+        dict.__init__(self, [(k, convert(v)) for k, v in data.items()])
+
+    def __getitem__(self, item):
+        try:
+            return dict.__getitem__(self, item)
+        except KeyError:
+            # Unlike a defaultdict, don't set the key, just return a dict.
+            return ScenarioDict({})
+
+
+class CompareType(object):
+    """Class that compares equal to any object of the given type."""
+    def __init__(self, type):
+        self.type = type
+
+    def __eq__(self, other):
+        return isinstance(other, self.type)
+
+    def __ne__(self, other):
+        """Needed for Python 2."""
+        return not self.__eq__(other)
+
+
+class FunctionCallRecorder(object):
+    """Utility class to wrap a callable and record its invocations."""
+    def __init__(self, function):
+        self._function = function
+        self._call_list = []
+
+    def __call__(self, *args, **kwargs):
+        self._call_list.append((args, kwargs))
+        return self._function(*args, **kwargs)
+
+    def reset(self):
+        """Wipes the call list."""
+        self._call_list = []
+
+    def call_list(self):
+        """Returns a copy of the call list."""
+        return self._call_list[:]
+
+    @property
+    def call_count(self):
+        """Returns the number of times the function has been called."""
+        return len(self._call_list)
+
+
+class TestCreator(object):
+    """Class to create test cases from specifications."""
+    def __init__(self, create_test, test_class, test_path):
+        """Create a TestCreator object.
+
+        :Parameters:
+          - `create_test`: callback that returns a test case. The callback
+            must accept the following arguments - a dictionary containing the
+            entire test specification (the `scenario_def`), a dictionary
+            containing the specification for which the test case will be
+            generated (the `test_def`).
+          - `test_class`: the unittest.TestCase class in which to create the
+            test case.
+          - `test_path`: path to the directory containing the JSON files with
+            the test specifications.
+            """
+        self._create_test = create_test
+        self._test_class = test_class
+        self.test_path = test_path
+
+    def _ensure_min_max_server_version(self, scenario_def, method):
+        """Test modifier that enforces a version range for the server on a
+        test case."""
+        if 'minServerVersion' in scenario_def:
+            min_ver = tuple(
+                int(elt) for
+                elt in scenario_def['minServerVersion'].split('.'))
+            if min_ver is not None:
+                method = client_context.require_version_min(*min_ver)(method)
+
+        if 'maxServerVersion' in scenario_def:
+            max_ver = tuple(
+                int(elt) for
+                elt in scenario_def['maxServerVersion'].split('.'))
+            if max_ver is not None:
+                method = client_context.require_version_max(*max_ver)(method)
+
+        return method
+
+    @staticmethod
+    def valid_topology(run_on_req):
+        return client_context.is_topology_type(
+            run_on_req.get('topology', ['single', 'replicaset', 'sharded']))
+
+    @staticmethod
+    def min_server_version(run_on_req):
+        version = run_on_req.get('minServerVersion')
+        if version:
+            min_ver = tuple(int(elt) for elt in version.split('.'))
+            return client_context.version >= min_ver
+        return True
+
+    @staticmethod
+    def max_server_version(run_on_req):
+        version = run_on_req.get('maxServerVersion')
+        if version:
+            max_ver = tuple(int(elt) for elt in version.split('.'))
+            return client_context.version <= max_ver
+        return True
+
+    def should_run_on(self, scenario_def):
+        run_on = scenario_def.get('runOn', [])
+        if not run_on:
+            # Always run these tests.
+            return True
+
+        for req in run_on:
+            if (self.valid_topology(req) and
+                    self.min_server_version(req) and
+                    self.max_server_version(req)):
+                return True
+        return False
+
+    def ensure_run_on(self, scenario_def, method):
+        """Test modifier that enforces a 'runOn' on a test case."""
+        return client_context._require(
+            lambda: self.should_run_on(scenario_def),
+            "runOn not satisfied",
+            method)
+
+    def tests(self, scenario_def):
+        """Allow CMAP spec test to override the location of test."""
+        return scenario_def['tests']
+
+    def create_tests(self):
+        for dirpath, _, filenames in os.walk(self.test_path):
+            dirname = os.path.split(dirpath)[-1]
+
+            for filename in filenames:
+                with open(os.path.join(dirpath, filename)) as scenario_stream:
+                    # Use tz_aware=False to match how CodecOptions decodes
+                    # dates.
+                    opts = json_util.JSONOptions(tz_aware=False)
+                    scenario_def = ScenarioDict(
+                        json_util.loads(scenario_stream.read(),
+                                        json_options=opts))
+
+                test_type = os.path.splitext(filename)[0]
+
+                # Construct test from scenario.
+                for test_def in self.tests(scenario_def):
+                    test_name = 'test_%s_%s_%s' % (
+                        dirname,
+                        test_type.replace("-", "_").replace('.', '_'),
+                        str(test_def['description'].replace(" ", "_").replace(
+                            '.', '_')))
+
+                    new_test = self._create_test(
+                        scenario_def, test_def, test_name)
+                    new_test = self._ensure_min_max_server_version(
+                        scenario_def, new_test)
+                    new_test = self.ensure_run_on(
+                        scenario_def, new_test)
+
+                    new_test.__name__ = test_name
+                    setattr(self._test_class, new_test.__name__, new_test)
+
+
+def _connection_string(h, authenticate):
     if h.startswith("mongodb://"):
         return h
     elif client_context.auth_enabled and authenticate:
-        return "mongodb://%s:%s@%s:%d" % (db_user, db_pwd, str(h), p)
+        return "mongodb://%s:%s@%s" % (db_user, db_pwd, str(h))
     else:
-        return "mongodb://%s:%d" % (str(h), p)
+        return "mongodb://%s" % (str(h),)
 
 
 def _mongo_client(host, port, authenticate=True, direct=False, **kwargs):
@@ -139,7 +419,7 @@ def _mongo_client(host, port, authenticate=True, direct=False, **kwargs):
         client_options['replicaSet'] = client_context.replica_set_name
     client_options.update(kwargs)
 
-    client = MongoClient(_connection_string(host, port, authenticate), port,
+    client = MongoClient(_connection_string(host, authenticate), port,
                          **client_options)
 
     return client
@@ -181,15 +461,39 @@ def rs_or_single_client(h=None, p=None, **kwargs):
     return _mongo_client(h, p, **kwargs)
 
 
+def ensure_all_connected(client):
+    """Ensure that the client's connection pool has socket connections to all
+    members of a replica set. Raises ConfigurationError when called with a
+    non-replica set client.
+
+    Depending on the use-case, the caller may need to clear any event listeners
+    that are configured on the client.
+    """
+    ismaster = client.admin.command("isMaster")
+    if 'setName' not in ismaster:
+        raise ConfigurationError("cluster is not a replica set")
+
+    target_host_list = set(ismaster['hosts'])
+    connected_host_list = set([ismaster['me']])
+    admindb = client.get_database('admin')
+
+    # Run isMaster until we have connected to each host at least once.
+    while connected_host_list != target_host_list:
+        ismaster = admindb.command("isMaster",
+                                   read_preference=ReadPreference.SECONDARY)
+        connected_host_list.update([ismaster["me"]])
+
+
 def one(s):
     """Get one element of a set"""
     return next(iter(s))
 
 
-def oid_generated_on_client(oid):
-    """Is this process's PID in this ObjectId?"""
-    pid_from_doc = struct.unpack(">H", oid.binary[7:9])[0]
-    return (os.getpid() % 0xFFFF) == pid_from_doc
+def oid_generated_on_process(oid):
+    """Makes a determination as to whether the given ObjectId was generated
+    by the current process, based on the 5-byte random number in the ObjectId.
+    """
+    return ObjectId._random() == oid.binary[4:9]
 
 
 def delay(sec):
@@ -202,6 +506,38 @@ def get_command_line(client):
     return command_line
 
 
+def camel_to_snake(camel):
+    # Regex to convert CamelCase to snake_case.
+    snake = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', camel)
+    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', snake).lower()
+
+
+def camel_to_upper_camel(camel):
+    return camel[0].upper() + camel[1:]
+
+
+def camel_to_snake_args(arguments):
+    for arg_name in list(arguments):
+        c2s = camel_to_snake(arg_name)
+        arguments[c2s] = arguments.pop(arg_name)
+    return arguments
+
+
+def parse_collection_options(opts):
+    if 'readPreference' in opts:
+        opts['read_preference'] = parse_read_preference(
+            opts.pop('readPreference'))
+
+    if 'writeConcern' in opts:
+        opts['write_concern'] = WriteConcern(
+            **dict(opts.pop('writeConcern')))
+
+    if 'readConcern' in opts:
+        opts['read_concern'] = ReadConcern(
+            **dict(opts.pop('readConcern')))
+    return opts
+
+
 def server_started_with_option(client, cmdline_opt, config_opt):
     """Check if the server was started with a particular option.
 
@@ -267,9 +603,10 @@ def server_is_master_with_slave(client):
 
 
 def drop_collections(db):
-    for coll in db.list_collection_names():
-        if not coll.startswith('system'):
-            db.drop_collection(coll)
+    # Drop all non-system collections in this database.
+    for coll in db.list_collection_names(
+            filter={"name": {"$regex": r"^(?!system\.)"}}):
+        db.drop_collection(coll)
 
 
 def remove_all_users(db):
@@ -281,7 +618,7 @@ def joinall(threads):
     """Join threads with a 5-minute timeout, assert joins succeeded"""
     for t in threads:
         t.join(300)
-        assert not t.isAlive(), "Thread %s hung" % t
+        assert not t.is_alive(), "Thread %s hung" % t
 
 
 def connected(client):
@@ -309,6 +646,7 @@ def wait_until(predicate, success_description, timeout=10):
     Returns the predicate's first true value.
     """
     start = time.time()
+    interval = min(float(timeout)/100, 0.1)
     while True:
         retval = predicate()
         if retval:
@@ -317,7 +655,7 @@ def wait_until(predicate, success_description, timeout=10):
         if time.time() - start > timeout:
             raise AssertionError("Didn't ever %s" % success_description)
 
-        time.sleep(0.1)
+        time.sleep(interval)
 
 
 def is_mongos(client):
@@ -375,78 +713,6 @@ class DeprecationFilter(object):
         self.warn_context = None
 
 
-def read_from_which_host(
-        client,
-        pref,
-        tag_sets=None,
-):
-    """Read from a client with the given Read Preference.
-
-    Return the 'host:port' which was read from.
-
-    :Parameters:
-      - `client`: A MongoClient
-      - `mode`: A ReadPreference
-      - `tag_sets`: List of dicts of tags for data-center-aware reads
-    """
-    db = client.pymongo_test
-
-    if isinstance(tag_sets, dict):
-        tag_sets = [tag_sets]
-    if tag_sets:
-        tags = tag_sets or pref.tag_sets
-        pref = pref.__class__(tags)
-
-    db.read_preference = pref
-
-    cursor = db.test.find()
-    try:
-        try:
-            next(cursor)
-        except StopIteration:
-            # No documents in collection, that's fine
-            pass
-
-        return cursor.address
-    except AutoReconnect:
-        return None
-
-
-def assertReadFrom(testcase, client, member, *args, **kwargs):
-    """Check that a query with the given mode and tag_sets reads from
-    the expected replica-set member.
-
-    :Parameters:
-      - `testcase`: A unittest.TestCase
-      - `client`: A MongoClient
-      - `member`: A host:port expected to be used
-      - `mode`: A ReadPreference
-      - `tag_sets` (optional): List of dicts of tags for data-center-aware reads
-    """
-    for _ in range(10):
-        testcase.assertEqual(member,
-                             read_from_which_host(client, *args, **kwargs))
-
-
-def assertReadFromAll(testcase, client, members, *args, **kwargs):
-    """Check that a query with the given mode and tag_sets reads from all
-    members in a set, and only members in that set.
-
-    :Parameters:
-      - `testcase`: A unittest.TestCase
-      - `client`: A MongoClient
-      - `members`: Sequence of host:port expected to be used
-      - `mode`: A ReadPreference
-      - `tag_sets` (optional): List of dicts of tags for data-center-aware reads
-    """
-    members = set(members)
-    used = set()
-    for _ in range(100):
-        used.add(read_from_which_host(client, *args, **kwargs))
-
-    testcase.assertEqual(members, used)
-
-
 def get_pool(client):
     """Get the standalone, primary, or mongos pool."""
     topology = client._get_topology()
@@ -481,7 +747,7 @@ def run_threads(collection, target):
 
     for t in threads:
         t.join(60)
-        assert not t.isAlive()
+        assert not t.is_alive()
 
 
 @contextlib.contextmanager
@@ -570,3 +836,28 @@ def enable_replication(client):
         secondary = single_client(host, port)
         secondary.admin.command('configureFailPoint', 'stopReplProducer',
                                 mode='off')
+
+
+class ExceptionCatchingThread(threading.Thread):
+    """A thread that stores any exception encountered from run()."""
+    def __init__(self, *args, **kwargs):
+        self.exc = None
+        super(ExceptionCatchingThread, self).__init__(*args, **kwargs)
+
+    def run(self):
+        try:
+            super(ExceptionCatchingThread, self).run()
+        except BaseException as exc:
+            self.exc = exc
+            raise
+
+
+def parse_read_preference(pref):
+    # Make first letter lowercase to match read_pref's modes.
+    mode_string = pref.get('mode', 'primary')
+    mode_string = mode_string[:1].lower() + mode_string[1:]
+    mode = read_preferences.read_pref_mode_from_name(mode_string)
+    max_staleness = pref.get('maxStalenessSeconds', -1)
+    tag_sets = pref.get('tag_sets')
+    return read_preferences.make_read_preference(
+        mode, tag_sets=tag_sets, max_staleness=max_staleness)
\ No newline at end of file
